/**************************************************************************************************
	<<< Render Script of Cards >>>
***************************************************************************************************
	Parameters:
	Config.
        GeneratorFolder = Folder where generator is registered, resources are in 'images' sub-folder.
        ArtFolder = Folder where art is resided, each edition in a subfolder.
        IsPre8thFrame = Render in Pre-8th frame.
        ApplicationFolder = Current path where application runs.
	CardInfo = Associative array with the card info to be rendered
		Set, Name, Version, Color, Cost, Type, Text, PT, Flavor, Rarity
		Artist, Number, Mana, LandColor, GuildName, AltFrame
	Dimensions: w1050 h736	
**************************************************************************************************/
var ImageBuffer = new Array();

function Main(bmpBuffer, CardInfo) {
    try {
        var RenderedCard = RenderCard(CardInfo);
        try {
            var BitmapBuffer = RenderedCard.GetBitmap(Delphi.GPMakeColor(0xff, 0x00, 0xff));
            try {
                bmpBuffer.Assign(BitmapBuffer);    
            } finally {
                BitmapBuffer.Free();
            }  
        } finally {
            RenderedCard.Free();
        }
    } finally {
        for (var name in ImageBuffer) {
            ImageBuffer[name].Free();
        }
        ImageBuffer.length = 0;
    }
    return true;
} 

function RenderCard(CardInfo) {
    // Verificar o tipo de carta a ser gerada...
    var cardName = CardInfo.Values('Name').toLowerCase();
    var cardSet = CardInfo.Values('Set').toLowerCase();
    var cardType = CardInfo.Values('Type').toLowerCase();
    var cardText = Delphi.Trim(CardInfo.Values('Text')).toLowerCase();
	var cardFlavor = Delphi.Trim(CardInfo.Values('Flavor')).toLowerCase();
    var cardFrame = CardInfo.Values('AltFrame').toLowerCase();
    if (cardName.Countain('/')) {
        // Split Card!
        return RenderSplitCard(CardInfo);
    }
    if ((cardName.indexOf('avatar:') == 0) || (cardSet == 'vg')) {
        // Vanguard
        return RenderVanguardCard(CardInfo);
    }
    if (cardType.Countain('planeswalker')) {
        // Planeswalker
		return RenderPlaneswalkerCard(CardInfo);
    }
    if (cardText.Countain('----')) {
        // Flip
		return RenderFlipCard(CardInfo, cardText.Countain('-----'));
    }
    if (cardFrame == 'timeshifted') {
        // TimeShifted (Work Same as Normal)
        return RenderNormalCard(CardInfo);
    } 
    if (cardFrame == 'futureshifted') {
        // FutureShifted
		return RenderFutureCard(CardInfo, (cardText == '') && (cardFlavor == ''));
    }
	// Token
	if ((flagType = (cardType.substr(0, 6) == 'token ')) || (cardSet == 'tk') || (cardSet == 'tkx')) {
		if (flagType) {
			CardInfo.Values('Type') = CardInfo.Values('Type').replace(/^token /gi, '');
		}
		return RenderTokenCard(CardInfo);
	}
    // Normal Card
    return RenderNormalCard(CardInfo);
}

/* Prototype */
String.prototype.Countain = function(partStr) {
	return (partStr) && (this.indexOf(partStr) != -1);
}
String.prototype.IsSameOf = function(compareStr) {
	return this.toLowerCase() == compareStr.toLowerCase();
}

/* Internal Functions/Constants */
var ColorTable = ['', 'W', 'U', 'WU', 'B', 'WB', 'UB', 'WUB',
        'R', 'RW', 'UR', 'RWU', 'BR', 'WBR', 'UBR', 'WUBR',
        'G', 'GW', 'GU', 'GWU', 'BG', 'BGW', 'GUB', 'GWUB',
        'RG', 'RGW', 'URG', 'RGWU', 'BRG', 'BRGW', 'UBRG',
        'WUBRG'];
function GetColors(castingCost) {
    var J = 0;
    for(var i = 0; i < castingCost.length; i++) {
        switch (castingCost.charAt(i)) {
        case 'W': J |= 0x01; break;
        case 'U': J |= 0x02; break;
        case 'B': J |= 0x04; break;
        case 'R': J |= 0x08; break;
        case 'G': J |= 0x10; break;
        case 'A': return 'A';
        case 'C': return 'C';
        case '*': return ColorTable[J];
        }
    }
    return ColorTable[J];
}
function GetDualColors(castingCost) {
    var result = GetColors(castingCost);
    if (!Config.MultiColorCardFrame.Countain(result.length.toString())) {
		if (result.length == 2) {
			for(var i = 0; i < ColorTable.length; i++) {
				if (ColorTable[i].length == 2) {
					if (castingCost.Countain('{'+ColorTable[i]+'}')) {
						return result;
					}
				}
			}
		}
        return "";
    } else {
        return result;
    }
}
function GetImage(fileName) {
    if (ImageBuffer[fileName]) {
        return ImageBuffer[fileName];
    }
    var fullFileName = Config.GeneratorFolder + fileName;
    var buffer = CreateObject('TGPBitmap', fullFileName);
    if (buffer.LastStatus == GPStatusOk) {
        ImageBuffer[fileName] = buffer;
        return buffer;
    } else {
		buffer.Free();
	}  
    return false;
}
function GetDirectImage(filename) {
    var buffer = CreateObject('TGPBitmap', filename);
    if (buffer.LastStatus == GPStatusOk) {
        return buffer;
    } 
    return false;
}
function GetSymbol(symbolName) {
    return GetImage('images\\symbols\\' + symbolName + '.png');
}
function GetFutureSymbol(symbolName) {
    return GetImage('images\\eighth\\futureshifted\\symbols\\' + symbolName + '.png');
}
function PreCastingCost(castingCost) {
    if ((castingCost.length = 0) || (castingCost.charAt(0) == '{')) {
        return castingCost;
    }
    var result = "";
    for (var i = 0; i < castingCost.length; i++) {
        var costChar = castingCost.charAt(i).toUpperCase();
        switch (costChar) {
        case "W":        case "U":        case "B":        case "R":
        case "G":        case "S":        case "A":        case "C":
        case "X":        case "Y":        case "Z":        case "0":
        case "2":        case "3":        case "4":        case "5":
        case "6":        case "7":        case "8":        case "9":
            result += "{" + costChar + "}";
            break;
        case "/":        case "*":
            result += costChar;
        case "1":
            var twoChar = castingCost.charAt(i + 1);
            if ((twoChar.length == 1) && "01256".Countain(twoChar)) {
                result += "{1" + twoChar + "}";
                i++;
            } else {
                result += "{1}";
            }     
            break;
        case "{":
            for (var j = i + 1; (i < castingCost.length) && (castingCost.charAt(i) != "}"); i++) {}
            result += "{" + castingCost.substr(j, i - j) + "}";
            break;
        }
    }
    return result;
}
function Iif(cond, vt, vf) {
    return (cond)?vt:vf;
}
function IfPre8th(vt, vf) {
	return (Config.IsPre8thFrame)?vt:vf;
}
function Max(v1, v2) {
    return Iif(v1 > v2, v1, v2);
}
function HaveTombStone(Card) {
    return Config.IsPre8thFrame &&
        (
            Card.Values('Name').IsSameOf('Ichorid') ||  
            Card.Values('Name').IsSameOf('Riftstone Portal') ||
            Card.Values('Type').IsSameOf('Creature - Incarnation') ||
            Card.Values('Text').toLowerCase().Countain('flashback')
        );
}
function StrSplit(strValue, strSep) {
	// TODO: Handle Recursive Separators... Up To 6...
    var arr = strValue.split(strSep);
    switch (arr.length) {
    case 0: return ['', ''];
    case 1: return [arr[0], arr[0]];
    default:
		return [arr[0], arr[1]];
    }
}
function SplitCardInfo(Source, StrKey, StrSep, Dest1, Dest2) {
    var arr = StrSplit(Source.Values(StrKey), StrSep);
    Dest1.Values(StrKey) = arr[0];
    Dest2.Values(StrKey) = arr[1];    
}
function Debug(Message) {
    return Delphi.ShowMessage(Message, 'Debug');
}

/* Render a simple and one card info... */
function RenderNormalCard(Card) {
    var workBuffer = CreateObject('TGPBitmap', 736, 1050, GPPixelFormat32bppARGB);
    var canvas = workBuffer.GetGraphics();
    try {
        canvas.SetInterpolationMode(GPInterpolationModeHighQualityBicubic);
        canvas.SetTextRenderingHint(GPTextRenderingHintAntiAlias);
        //
        var cardSet = Card.Values('Set');
        var cardName = Card.Values('Name');
        var cardVersion = Card.Values('Version');
        var cardColor = Card.Values('Color');
        var cardCost = PreCastingCost(Card.Values('Cost'));
        var cardType = Card.Values('Type').replace(/ - /g, '  ');
        var cardText = Delphi.Trim(Card.Values('Text')).replace(/~this~/g, cardName).replace(/ - /g, '  ');
        var cardPT = Card.Values('PT');
        var cardFlavor = Delphi.Trim(Card.Values('Flavor')).replace(/ - /g, '  ');
        var cardRarity = Card.Values('Rarity');
        var cardArtist = Card.Values('Artist');
        var cardNumber = Card.Values('Number');
        var cardMana = Card.Values('Mana');
        var cardLandColor = Delphi.Trim(Card.Values('LandColor'));
        var cardGuildName = Card.Values('GuildName');
        var cardAltFrame = Card.Values('AltFrame');
        var cardCopyright = Card.Values('Copyright').replace(/\(tm\)/g, '').replace(/\(c\)/g, '');
        var cardArt = Card.Values('Art');
        // Normalize LandColor
		cardLandColor = GetColors(PreCastingCost(cardLandColor));
		//
        var symbols = GetColors(cardCost);
        var dualColor = GetDualColors(cardCost);
        if (cardColor.length == 0) {
            if (cardLandColor.length > 0) {
                cardColor = "Lnd";
            } else if (symbols.length > 1) {
                cardColor = "Gld";
            } else if (symbols.length > 0) {
                cardColor = symbols;
            } else {
                cardColor = "Art";
            }  
        }
        var image;
        var borderImage = false;
        var overlay = false; 
        if (cardColor == "Lnd") {
            if (cardLandColor.length = 0) {
                throw new Error("Land color missing for card: " + cardName);
            }
            if ((cardLandColor.length > 2) && (!Config.MultiColorLandFrame)) {
                cardLandColor = "A";
            }
            if ((cardLandColor.length > 2) && (Config.IsPre8thFrame)) {
				// Old Frames Doesn't Support 3 or more colors...
                cardLandColor = "A";
            }
            if (Config.MultiColorFetchLandFrame) {
                switch (cardName.toUpperCase()) {
                case "FLOODED STRAND":      case "FLOOD PLAIN":     cardLandColor = "WU"; break; 
                case "BLOODSTAINED MIRE":   case "ROCKY TAR PIT":   cardLandColor = "BR"; break; 
                case "WOODED FOOTHILLS":    case "MOUNTAIN VALLEY": cardLandColor = "RG"; break; 
                case "POLLUTED DELTA":      case "BAD RIVER":       cardLandColor = "UB"; break; 
                case "WINDSWEPT HEATH":     case "GRASSLANDS":      cardLandColor = "GW"; break; 
                }
            }
            if (Config.IsPre8thFrame) {
                image = GetImage("images\\preEighth\\land\\" + cardLandColor + ".png"); 
            } else {
                image = GetImage("images\\eighth\\regular\\land\\" + cardLandColor + ".png"); 
            }
            if (image === false) {
                throw new Error("Background image not found for land color: " + cardLandColor);
            } 
        } else {
            if ((!Config.IsPre8thFrame) && (dualColor != "")) {
                image = GetImage("images\\eighth\\regular\\cards\\" + dualColor + ".png");
                if (image === false) {
                    throw new Error("Background image not found for color: " + dualColor);
                }
                if (Config.BlendGreyTitleAndType) {
                    overlay = GetImage("images\\eighth\\regular\\cards\\C-overlay.png");
                    if (overlay === false) {
                        throw new Error("Image not found: C-overlay.png");
                    } 
                }
            } else {
                if (cardAltFrame != "") {
                    image = GetImage("images\\eighth\\" + cardAltFrame + "\\cards\\" + cardColor + ".png");
                } else {
                    if (Config.IsPre8thFrame) {
                        image = GetImage("images\\preEighth\\cards\\" + cardColor + ".png");
                    } else {
                        image = GetImage("images\\eighth\\regular\\cards\\" + cardColor + ".png");
                    }                    
                }
                if (image === false) {
                    throw new Error("Background image not found for color: " + cardColor);
                }
                if ((!Config.IsPre8thFrame) && (symbols.length == 2) && (Config.MultiColorDualBorder)) {
                    borderImage = GetImage("images\\eighth\\regular\\borders\\" + symbols + ".png");
                }
            }
        }
        // Art (Direct Handed Full...)
        var cardArtFull = '';
        if (cardArt.Countain('.full.')) {
            cardArtFull = cardArt;
            cardArt = ''; 
        }
        if (cardArt.length == 0) {
            var cleanTitle = cardName.toLowerCase().replace(/[:\/]/g, '');
            var list = Delphi.Directory(Config.ArtFolder + cardSet + "\\" + cleanTitle + "*.jpg");
            try {
                for(var i = list.Count; i > 0; i--) {
                    var j = i - 1;
                    var file = list.Strings(j).toLowerCase();					
					var chrPos = file.indexOf(cleanTitle) + cleanTitle.length;
                    var chr = file.charAt(chrPos);
                    if (file.indexOf(".full.jpg") == chrPos) { // .full.jpg must be at end of name...
                        cardArtFull = list.Strings(j);
                    }
                    if (file.Countain(".full.jpg") || !".(123456789".Countain(chr)) {
                        list.Delete(j);
                    }
                }
                if (list.Count > 0) {
                    cardArt = list.Strings(Delphi.Random(list.Count)); 
                }
            } finally {
                list.Free();    
            }
        }
        var ArtImage;
        ArtImage = GetDirectImage(cardArt);
        if (!ArtImage === false) {
            var ArtLeft = IfPre8th(67, 42);
            var ArtRight = IfPre8th(669, 694);
            var ArtTop = IfPre8th(82, 104);
            var ArtBottom = IfPre8th(573, 584);
            var ArtNewW = ArtRight - ArtLeft + 1;
            var ArtNewH = ArtBottom - ArtTop + 1;
            var ArtW = ArtNewW;
            var ArtH = ArtNewH;
            var saveState = canvas.Save();
            try {
                canvas.SetClip(ArtLeft, ArtTop, ArtW, ArtH);
                ArtH = ArtImage.GetHeight() * (ArtNewW / ArtImage.GetWidth());
                if (ArtH < ArtNewH) {
                    ArtH = ArtNewH;
                    ArtW = ArtImage.GetWidth() * (ArtNewH / ArtImage.GetHeight());
                }
                canvas.DrawImage(ArtImage, ArtLeft + ((ArtNewW - ArtW) / 2), ArtTop + ((ArtNewH - ArtH) / 2), ArtW, ArtH);
            } finally {
                canvas.Restore(saveState);
            }
            ArtImage.Free();
        } else if (cardArtFull.length > 0) {
            ArtImage = GetDirectImage(cardArtFull);
            if (!ArtImage === false) {
                canvas.DrawImage(ArtImage, 0, 0, 736, 1050);
                ArtImage.Free();
            }
        }
        // Special Frames
        if (cardName.IsSameOf("Murmuring Bosk")) {
            overlay = GetImage("images\\eighth\\regular\\cards\\C-overlay-G.png");
            if (overlay === false) {
                throw new Error("Image not found: C-overlay-G.png");
            } 
        }
		// If a "Artifact" have mana cost with colors, render the superb border over it!
		// Examples:
		//  Reaper King = {2W}{2U}{2B}{2R}{2G}
		//  Sarcomite Myr = {2}{U}
		//  Tranguild Courier = {W}{U}{B}{R}{G}*{4}
		var cardColorByCost = "";
		if (cardColor == "Art") {
			cardColorByCost = symbols;
			if (cardColorByCost.length > 2) {
				cardColorByCost = "Gld";
			}
			if (cardColorByCost.length > 0) { 
				borderImage = GetImage("images\\eighth\\regular\\borders\\" + cardColorByCost + ".png");
				if (borderImage === false) {
					throw new Error("Image not found: " + cardColorByCost + ".png");
				} 
			}
		}
        // Bg
        canvas.DrawImage(image, 0, 0, 736, 1050);
        if (!borderImage === false) {
            canvas.DrawImage(borderImage, 0, 0, 736, 1050);
        }
        if (!overlay === false) {
            canvas.DrawImage(overlay, 0, 0, 736, 1050);
        }
        // Power/Toughness
        if (cardPT.length > 0) {
            if (!Config.IsPre8thFrame) {
                if (dualColor == "") {
                    if ((cardColor.length > 1) && (cardColor != "Art") && (cardColor != "Gld")) {
                        image = GetImage("images\\eighth\\regular\\pt\\" + cardColor.substr(-1) + ".png");    
                    } else {
                    if (cardAltFrame != "") {
                            image = GetImage("images\\eighth\\" + cardAltFrame + "\\pt\\" + cardColor + ".png");
                        } else {
                            image = GetImage("images\\eighth\\regular\\pt\\" + cardColor + ".png");                
                        }
                    }
                } else {
                    var OneColor = dualColor.charAt(dualColor.length - 1);  
                    image = GetImage("images\\eighth\\regular\\pt\\" + OneColor + ".png");
                }
                if (image === false) {
                    throw new Error("Power / toughness image not found for color: " + cardColor);
                }
                canvas.DrawImage(image, 0, 1050 - image.GetHeight(), image.GetWidth(), image.GetHeight());
            } 
            // Render Text...
            var fontPT = IfPre8th("MPlantinB.ttf", "GaramondB.ttf");
            var renderer = canvas.GetTextRenderer(Config.GeneratorFolder, fontPT, fontPT);
            try {
                var measure = { "X":0, "Y":0 };
                renderer.Size = 36;
                renderer.Color = Delphi.GPMakeColor(0x00, 0x00, 0x00);
                renderer.TextPowTou = cardPT;
                while ((renderer.Measure(measure)) && (measure.X >= 118)) {
                    renderer.Size -= 0.1;
                    if (renderer.Size < 8) {
                        throw new Error("Power/toughness does not fit on card: " + cardName);
                    }
                }
                if ((Config.IsPre8thFrame) || (cardAltFrame != "")) {
                    renderer.Color = Delphi.GPMakeColor(0x00, 0x00, 0x00);
                    if (cardAltFrame != "") {
                        renderer.Render(625, 982, GPStringAlignmentCenter, GPStringAlignmentCenter);
                    } else {
                        renderer.Render(629, 982, GPStringAlignmentCenter, GPStringAlignmentCenter);
                    }
                    renderer.Color = Delphi.GPMakeColor(0xff, 0xff, 0xff);             
                }
                renderer.TextPowTou = cardPT;
                renderer.Render(627, 980, GPStringAlignmentCenter, GPStringAlignmentCenter);
            } finally {
                renderer.Free();
            }
        }
        // CastingCost...
        var costList = CreateObject("TStringList");
        try {
            var costSep = cardCost.indexOf("*");
            var SizeY;
            var DesY;
            var CCRight = 696;
            var CCTop = IfPre8th(32, 46);
            if (costSep == -1) {
                costList.CommaText = cardCost.replace(/}{/g, '","').replace(/{/g, '"').replace(/}/g, '"');      
            } else {
                costList.CommaText = cardCost.substr(costSep + 1).replace(/}{/g, '","').replace(/{/g, '"').replace(/}/g, '"');          
            } 
            for (var i = 1; i <= costList.Count; i++) {
                var oneItem = costList.Strings(costList.Count - i);
                if ((oneItem.length == 2) && !"10,11,12,15,16".Countain(oneItem)) {
                    SizeY = 42;
                    DesY = -4;
                } else {
                    SizeY = 34;
                    DesY = 0;
                }
                image = GetSymbol(oneItem);
				if (image === false) {
					throw new Error("Symbol not found: " + oneItem);
				}				
                SizeX = SizeY * (image.GetWidth() / image.GetHeight());
                CCRight -= SizeX;
                canvas.DrawImage(GetSymbol('shadow'), CCRight - 3, CCTop + DesY + 3, SizeX + 2, SizeY + 1);
                canvas.DrawImage(image, CCRight, CCTop + DesY, SizeX, SizeY);
                CCRight -= 3;    
            } 
            CCRight -= 5;
        } finally {
            costList.Free();
        }
        // CardTitle
        var fontTitle = IfPre8th("Pre8th.ttf", "GaramondB.ttf");
        var renderer = canvas.GetTextRenderer(Config.GeneratorFolder, fontTitle, fontTitle);
        try {
            var measure = { "X":0, "Y":0 };
            var TitleLeft = 43;
            var TitleTop = IfPre8th(30, 35);
            if (HaveTombStone(Card)) {
                image = GetImage('images\\preEighth\\tombstone.png');
                if (!image === false) { 
                    canvas.DrawImage(image, TitleLeft - 20, TitleTop, 32, 45);
                    TitleLeft += 20;
                }
            }
            renderer.Size = 35;
            renderer.Color = Delphi.GPMakeColor(0x00, 0x00, 0x00);
            renderer.TextPowTou = cardName;
            while ((renderer.Measure(measure)) && (TitleLeft + measure.X >= CCRight)) {
                renderer.Size -= 0.1;
                if (renderer.Size < 8) {
                    throw new Error("Card title does not fit on card: " + cardName);
                }
            }
            if ((Config.IsPre8thFrame) || (cardAltFrame != "")) {
                renderer.Color = Delphi.GPMakeColor(0x00, 0x00, 0x00);
                renderer.Render(TitleLeft + 2, TitleTop + 2, GPStringAlignmentNear, GPStringAlignmentNear);
                renderer.Color = Delphi.GPMakeColor(0xff, 0xff, 0xff);             
            }                                                                                                       
            renderer.TextPowTou = cardName;
            renderer.Render(TitleLeft, TitleTop, GPStringAlignmentNear, GPStringAlignmentNear);
        } finally {
            renderer.Free();
        }
        // Rarity
        var rarityTop = IfPre8th(607, 622);
        var rarityRight = IfPre8th(680, 686);
        var rarityFolder = "images\\" + IfPre8th("preEighth", "eighth") + "\\rarity\\";
        image = GetImage(rarityFolder + cardSet + "_" + cardRarity + ".gif");
		if (image === false) {
			image = GetImage(rarityFolder + cardSet + "_" + cardRarity + ".png");
		}		
        if (!image === false) {
            var SizeX = image.GetWidth();
            var SizeY = image.GetHeight();
            if (image.GetHeight() > 38) {
                SizeX = 38 * (image.GetWidth() / image.GetHeight()); 
                SizeY = 38;
            }
            if (SizeX > 66) {
                SizeX = 66;
                SizeY = 66 * (image.GetHeight() / image.GetWidth());
            }
            rarityRight -= SizeX;
            canvas.DrawImage(image, rarityRight, rarityTop - (SizeY / 2), SizeX, SizeY);
            rarityRight -= 5;
        }    
        // Type
        var fontType = IfPre8th("MPlantin.ttf", "GaramondB.ttf");
        renderer = canvas.GetTextRenderer(Config.GeneratorFolder, fontType, fontType);
        try {
            measure = { "X":0, "Y":0 };
            var TypeLeft = 54;
            var TypeTop = IfPre8th(602, 618);
            renderer.Size = 30;
            renderer.Color = Delphi.GPMakeColor(0x00, 0x00, 0x00);
            renderer.TextPowTou = cardType;
            while ((renderer.Measure(measure)) && (TypeLeft + measure.X >= rarityRight)) {
                renderer.Size -= 0.1;
                if (renderer.Size < 8) {
                    throw new Error("Card type does not fit on card: " + cardName);
                }
            }
            if ((Config.IsPre8thFrame) || (cardAltFrame != "")) {
                renderer.Color = Delphi.GPMakeColor(0x00, 0x00, 0x00);
                renderer.Render(TypeLeft + 2, TypeTop + 2, GPStringAlignmentCenter, GPStringAlignmentNear);
                renderer.Color = Delphi.GPMakeColor(0xff, 0xff, 0xff);             
            }                                                                                                       
            renderer.TextPowTou = cardType;
            renderer.Render(TypeLeft, TypeTop, GPStringAlignmentCenter, GPStringAlignmentNear);
        } finally {
            renderer.Free();
        }    
        // Guild/Land Symbol
        if (cardGuildName) {
            image = GetImage("images\\eighth\\guilds\\" + cardGuildName + ".png");
            if (image === false) {
				image = GetImage("images\\promo\\" + cardGuildName + ".png");
				if (image === false) {
					throw new Error("Guild sign image not found for: " + cardGuildName);
				}
            }
            canvas.DrawImage(image, 373 - (image.GetWidth() / 2), IfPre8th(630, 650), image.GetWidth(), image.GetHeight());
        }
        //
        if (cardType.toLowerCase().Countain("basic land") || cardType.toLowerCase().Countain("basic snow land")) {
            image = GetImage("images\\symbols\\land\\" + cardLandColor + ".png");
            if (image === false) {
                throw new Error("Basic land symbol not found for: " + cardLandColor);
            }
            canvas.DrawImage(image, 368 - (image.GetWidth() / 2), IfPre8th(940, 960) - image.GetHeight(), image.GetWidth(), image.GetHeight());    
        } else if ((cardColor == "Lnd") && (cardLandColor.length == 2) && (cardText.length == 0)) {
            switch (Config.OutputDualLandSymbols) {
            case 1:
                image = GetImage("images\\symbols\\land\\" + cardLandColor + ".png");
                if (image === false) {
                    throw new Error("Merged land symbol not found for: " + cardLandColor);
                }
                canvas.DrawImage(image, 368 - (image.GetWidth() / 2), IfPre8th(920, 940) - image.GetHeight(), image.GetWidth(), image.GetHeight());
                break;           
            case 2:
                var image1 = GetImage("images\\symbols\\land\\" + cardLandColor.charAt(0) + ".png"); 
                var image2 = GetImage("images\\symbols\\land\\" + cardLandColor.charAt(1) + ".png");
                if ((image1 === false) || (image2 === false)) {
                    throw new Error("Basic land symbol not found for: " + cardLandColor);
                }
				var baseLine = IfPre8th(940, 960);
                var totalWidth = image1.GetWidth() + image2.GetWidth() + 10;
                var drawLeft = 368 - (totalWidth / 2);
                canvas.DrawImage(image1, drawLeft, baseLine - image1.GetHeight(), image1.GetWidth(), image1.GetHeight());
                drawLeft += image1.GetWidth() + 10;
                canvas.DrawImage(image2, drawLeft, baseLine - image2.GetHeight(), image2.GetWidth(), image2.GetHeight());
            }
        } else {
            var textHeight = IfPre8th(296, Iif(cardPT.length > 0, 276, 284));
            var textLeft = IfPre8th(73, 50);
            var textRight = IfPre8th(656, 682);
            var textTop = IfPre8th(638, 665);
            var textWidth = textRight - textLeft + 1;
            var textMeasure = { "X":0, "Y":0 };
            var flavorMeasure = { "X":0, "Y":0 };
            var totalHeight = 0;
            if ((cardText.length > 0) && (cardFlavor.length > 0)) {
                var textRenderer = canvas.GetTextRenderer(Config.GeneratorFolder, "MPlantin.ttf", "MPlantinI.ttf");
                try { 
                    textRenderer.MaxWidth = textWidth;
                    textRenderer.Size = 30; 
                    textRenderer.Color = Delphi.GPMakeColor(0x00, 0x00, 0x00);
                    textRenderer.Text = cardText; 
                    var flavorRenderer = canvas.GetTextRenderer(Config.GeneratorFolder, "MPlantin.ttf", "MPlantinI.ttf");
                    try { 
                        flavorRenderer.MaxWidth = textWidth;
                        flavorRenderer.Size = 27; 
                        flavorRenderer.Color = Delphi.GPMakeColor(0x00, 0x00, 0x00);
                        flavorRenderer.Text = "#" + cardFlavor; 
                        while (true) {
                            textRenderer.Measure(textMeasure);
                            flavorRenderer.Measure(flavorMeasure);
                            totalHeight = textMeasure.Y + flavorMeasure.Y + 20;
                            //
                            var totalWidth = Max(textMeasure.X, flavorMeasure.X); 
                            if ((totalHeight < textHeight) && (totalWidth < textWidth)) {
                                break;
                            }   
                            textRenderer.Size -= 0.1;
                            flavorRenderer.Size -= 0.1;
                            if ((textRenderer.Size < 8) || (flavorRenderer.Size < 8)) {
                                throw new Error("Text/Flavor doesn't fit on card: " + cardName);
                            } 
                        }
                        textTop += (textHeight - totalHeight) / 2;
                        textRenderer.Render(textLeft, textTop, GPStringAlignmentNear, GPStringAlignmentNear);  
                        flavorRenderer.Render(textLeft, textTop + 20 + textMeasure.Y, GPStringAlignmentNear, GPStringAlignmentNear);
                    } finally {
                        flavorRenderer.Free();
                    }
                } finally {
                    textRenderer.Free();
                }
            }
            else if (cardText.length > 0) {
                var textRenderer = canvas.GetTextRenderer(Config.GeneratorFolder, "MPlantin.ttf", "MPlantinI.ttf");
                try { 
                    textRenderer.MaxWidth = textWidth;
                    textRenderer.Size = 30; 
                    textRenderer.Color = Delphi.GPMakeColor(0x00, 0x00, 0x00);
                    textRenderer.Text = cardText; 
                    while (true) {
                        textRenderer.Measure(textMeasure);
                        totalHeight = textMeasure.Y;
                        //
                        var totalWidth = textMeasure.X; 
                        if ((totalHeight < textHeight) && (totalWidth < textWidth)) {
                            break;
                        }   
                        textRenderer.Size -= 0.1;
                        if (textRenderer.Size < 8) {
                            throw new Error("Text/Flavor doesn't fit on card: " + cardName);
                        } 
                    }
                    textRenderer.IsCenter = (textRenderer.LineCount <= 2);
                    textTop += (textHeight - totalHeight) / 2;
                    textRenderer.Render(textLeft, textTop, GPStringAlignmentNear, GPStringAlignmentNear);
                } finally {
                    textRenderer.Free();
                }  
            }
            else if (cardFlavor.length > 0) {
                var flavorRenderer = canvas.GetTextRenderer(Config.GeneratorFolder, "MPlantin.ttf", "MPlantinI.ttf");
                try { 
                    flavorRenderer.MaxWidth = textWidth;
                    flavorRenderer.Size = 27; 
                    flavorRenderer.Color = Delphi.GPMakeColor(0x00, 0x00, 0x00);
                    flavorRenderer.Text = "#" + cardFlavor; 
                    while (true) {
                        flavorRenderer.Measure(flavorMeasure);
                        totalHeight = flavorMeasure.Y;
                        //
                        var totalWidth = flavorMeasure.X; 
                        if ((totalHeight < textHeight) && (totalWidth < textWidth)) {
                            break;
                        }   
                        flavorRenderer.Size -= 0.1;
                        if (flavorRenderer.Size < 8) {
                            throw new Error("Text/Flavor doesn't fit on card: " + cardName);
                        } 
                    }
                    flavorRenderer.IsCenter = (flavorRenderer.LineCount <= 2);
                    textTop += (textHeight - totalHeight) / 2;
                    flavorRenderer.Render(textLeft, textTop, GPStringAlignmentNear, GPStringAlignmentNear);
                } finally {
                    flavorRenderer.Free();
                }  
            }
        }
        // Artist+Copyright
        if ((cardArtist.length > 0) && (cardCopyright.length > 0)) {
            if (Config.IsPre8thFrame) {
                renderer = canvas.GetTextRenderer(Config.GeneratorFolder, "GaramondB.ttf", "GaramondB.ttf");
                try {
                    renderer.Size = 20;
                    renderer.Text = "Illus. " + cardArtist;
                    renderer.Measure({ "X": 0, "Y": 0 });
                    renderer.Color = Delphi.GPMakeColor(0x00, 0x00, 0x00);
                    renderer.Render(370, 982, GPStringAlignmentCenter, GPStringAlignmentCenter);
                    renderer.Color = Delphi.GPMakeColor(0xff, 0xff, 0xff);
                    renderer.Render(368, 980, GPStringAlignmentCenter, GPStringAlignmentCenter);
                } finally {
                    renderer.Free();
                }
                renderer = canvas.GetTextRenderer(Config.GeneratorFolder, "MPlantin.ttf", "MPlantin.ttf");
                try {
                    renderer.Size = 14;
                    renderer.Text = cardCopyright;
                    renderer.Measure({ "X": 0, "Y": 0 });
                    renderer.Color = Iif(cardColor == "W", Delphi.GPMakeColor(0x00, 0x00, 0x00), Delphi.GPMakeColor(0xff, 0xff, 0xff));
                    renderer.Render(368, 1005, GPStringAlignmentCenter, GPStringAlignmentCenter);
                } finally {
                    renderer.Free();
                }
            } else {
                // Setup Color...
                var ArtCopyColor = 0x00;
                if ((cardAltFrame.length > 0) && ((cardColor == "B") || (cardColor == "R"))) {
                    ArtCopyColor = 0xff; 
                } else if ((cardColor == "B") || (cardColor == "G") || (cardColor == "R") || (cardColor == "Lnd")) {
                    ArtCopyColor = 0xff;
                } else if ((cardColor == "Gld") && (dualColor.length == 2) && ("BGR".indexOf(dualColor.charAt(0)) > -1)) {
                    ArtCopyColor = 0xff;
				}
                image = GetImage("images\\eighth\\brush_" + Iif(ArtCopyColor == 0, "0,0,0", "255,255,255") + ".png");
                canvas.DrawImage(image, 37, 989, 52, 12);
                //
                renderer = canvas.GetTextRenderer(Config.GeneratorFolder, "GaramondB.ttf", "GaramondB.ttf");
                try {
                    renderer.Size = 22;
                    renderer.Text = cardArtist;
                    renderer.Measure({ "X": 0, "Y": 0 });
                    renderer.Color = Delphi.GPMakeColor(ArtCopyColor, ArtCopyColor, ArtCopyColor);
                    renderer.Render(94, 1004, GPStringAlignmentFar, GPStringAlignmentNear);
                } finally {
                    renderer.Free();
                }
                //
                renderer = canvas.GetTextRenderer(Config.GeneratorFolder, "MPlantin.ttf", "MPlantin.ttf");
                try {
                    renderer.Size = 13;
                    renderer.Text = cardCopyright;
                    renderer.Measure({ "X": 0, "Y": 0 });
                    renderer.Color = Delphi.GPMakeColor(ArtCopyColor, ArtCopyColor, ArtCopyColor);
                    renderer.Render(37, 1025, GPStringAlignmentFar, GPStringAlignmentNear);
                } finally {
                    renderer.Free();
                }
            }
        }
    } finally {
        canvas.Free();
    }
    return workBuffer;
}

/* Render a split and one card info... */
function RenderSplitCard(Card) {
    var workBuffer = CreateObject('TGPBitmap', 1050, 736, GPPixelFormat32bppARGB);
    var canvas = workBuffer.GetGraphics();
	try {
        canvas.SetInterpolationMode(GPInterpolationModeHighQualityBicubic);
        canvas.SetTextRenderingHint(GPTextRenderingHintAntiAlias);
		var brush = CreateObject('TGPSolidBrush', Delphi.GPMakeColor(0x00, 0x00, 0x00));
		try {
			canvas.FillRectangle(brush, 0, 0, 1050, 736);
		} finally {
			brush.Free();
		}
        var Card1 = CreateObject('TStringList');
        try {
            Card1.Assign(Card);
            var Card2 = CreateObject('TStringList');
            try {
                Card2.Assign(Card);
                SplitCardInfo(Card, 'Name', '/', Card1, Card2);
                SplitCardInfo(Card, 'Type', '/', Card1, Card2);
                SplitCardInfo(Card, 'Color', '/', Card1, Card2);
                SplitCardInfo(Card, 'Cost', '/', Card1, Card2);
                SplitCardInfo(Card, 'Text', '//', Card1, Card2);
                SplitCardInfo(Card, 'Flavor', '//', Card1, Card2);
                SplitCardInfo(Card, 'PT', '//', Card1, Card2);
                SplitCardInfo(Card, 'Artist', '/', Card1, Card2);
                SplitCardInfo(Card, 'Art', '|', Card1, Card2);
                SplitCardInfo(Card, 'Copyright', '|', Card1, Card2);
                SplitCardInfo(Card, 'GuildName', '|', Card1, Card2);
                SplitCardInfo(Card, 'LandColor', '/', Card1, Card2);
				SplitCardInfo(Card, 'Set', '/', Card1, Card2);
				SplitCardInfo(Card, 'Rarity', '/', Card1, Card2);
				SplitCardInfo(Card, 'AltFrame', '/', Card1, Card2);
                var RenderedCard1 = RenderCard(Card1);
                try {
                    canvas.DrawImage(RenderedCard1, 5, 0, 515, 736);
                } finally {
                    RenderedCard1.Free();
                } 
                var RenderedCard2 = RenderCard(Card2);
                try {
                    canvas.DrawImage(RenderedCard2, 530, 0, 515, 736);
                } finally {
                    RenderedCard2.Free();
                } 
            } finally {
                Card2.Free();
            }
        } finally {
            Card1.Free();
        }
    } finally {
        canvas.Free();
	}
    return workBuffer;
}

function RenderVanguardCard(Card) {
	// Special Fonts for Vanguard!
	var TitleFont = "GaramondB.ttf";
	var HandLifeFont = "MPlantinB.ttf";
	var TextFont = "MPlantin.ttf";
	var TextFontI = "MPlantinI.ttf"; 
	//	
    var workBuffer = CreateObject('TGPBitmap', 736, 1050, GPPixelFormat32bppARGB);
    var canvas = workBuffer.GetGraphics();
    try {
        canvas.SetInterpolationMode(GPInterpolationModeHighQualityBicubic);
        canvas.SetTextRenderingHint(GPTextRenderingHintAntiAlias);
		brush = CreateObject('TGPSolidBrush', Delphi.GPMakeColor(0x00, 0x00, 0x00));
		try {
			canvas.FillRectangle(brush, 0, 0, 736, 1050);
		} finally {
			brush.Free();
		}
        // Card Data...
        var cardVersion = Iif(Config.ModernVanguardFrame, "new", "old");
        var cardName = Card.Values('Name').replace(/^avatar: /gi, "");
        var cardSet = Card.Values('Set');
        var cardArt = Card.Values('Art');
        var cardArtFull = '';
        var cardText = Delphi.Trim(Card.Values('Text'));
        var cardFlavor = Delphi.Trim(Card.Values('Flavor'));
        var cardArtist = Card.Values('Artist');
        var cardCopyright = Card.Values('Copyright').replace(/\(tm\)/g, '').replace(/\(c\)/g, '').replace(/0\//g, '');
        // Art...
        if (cardArt.length == 0) {
            cardArt = Config.ArtFolder + cardSet + "\\" + cardName + ".jpg";
            cardArtFull = Config.ArtFolder + cardSet + "\\" + cardName + ".full.jpg";
        }
        var ArtImage;
        ArtImage = GetDirectImage(cardArt);
        if (!ArtImage === false) {
            var ArtLeft = Iif(cardVersion == "old", 72, 56);
            var ArtTop = Iif(cardVersion == "old", 88, 60);
            var ArtNewW = Iif(cardVersion == "old", 588, 620);
            var ArtNewH = Iif(cardVersion == "old", 524, 580);
            var ArtW = ArtNewW;
            var ArtH = ArtNewH;
            var saveState = canvas.Save();
            try {
                canvas.SetClip(ArtLeft, ArtTop, ArtW, ArtH);
                ArtH = ArtImage.GetHeight() * (ArtNewW / ArtImage.GetWidth());
                if (ArtH < ArtNewH) {
                    ArtH = ArtNewH;
                    ArtW = ArtImage.GetWidth() * (ArtNewH / ArtImage.GetHeight());
                }
                canvas.DrawImage(ArtImage, ArtLeft + ((ArtNewW - ArtW) / 2), ArtTop + ((ArtNewH - ArtH) / 2), ArtW, ArtH);
            } finally {
                canvas.Restore(saveState);
            }
            ArtImage.Free();
        } else if (cardArtFull.length > 0) {
            ArtImage = GetDirectImage(cardArtFull);
            if (!ArtImage === false) {
                if (cardVersion == "old") {
                    canvas.DrawImage(ArtImage, 0, 0, 736, 1050);
                } else {
                    canvas.DrawImage(ArtImage, -20, -30, 776, 1110);
                }
                ArtImage.Free();
            }
        }
        // Background (OVER the art...) 
        var image = GetImage("images\\vanguard\\avatar-" + cardVersion + ".png");
        canvas.DrawImage(image, 0, 0, 736, 1050);
        // Title...
        var renderer = canvas.GetTextRenderer(Config.GeneratorFolder, TitleFont, TitleFont);
        try {
            var measure = { "X":0, "Y":0 };
            var TitleLeft = Iif(cardVersion == "old", 368, 368);
            var TitleTop = Iif(cardVersion == "old", 82, 62);
            var TitleColor = Iif(cardVersion == "old", 0x00, 0xFF);
            var TitleMaxWidth = Iif(cardVersion == "old", 490, 330);
            renderer.Size = 40;
            renderer.Color = Delphi.GPMakeColor(TitleColor, TitleColor, TitleColor);
            renderer.Text = cardName;
            while (true) {
                renderer.Measure(measure);
                if (measure.X <= TitleMaxWidth) {
                    break;
                }
                renderer.Size -= 0.1;
                if (renderer.Size < 9) {
                    throw new Error("Title does not fit on card: " + cardName);
                }
            }
            renderer.Render(TitleLeft, TitleTop, GPStringAlignmentFar, GPStringAlignmentCenter);
        } finally {
            renderer.Free();
        }
        // Oracle, Hand and Life...
        var i, j;
        i = cardText.toLowerCase().indexOf('starting life:');
        if (i == -1) { throw new Error('Vanguard "Starting Life" info missing on card: ' + cardName); }
        var cardLife = Delphi.Trim(cardText.substr(i + 14)).replace(/\./g, '');
        cardText = cardText.substr(0, i);
        j = 25;
        i = cardText.toLowerCase().indexOf('starting & max hand size:');
        if (i == -1) {
            j = 26;
            i = cardText.toLowerCase().indexOf('starting & max. hand size:');
            if (i == -1) {
                j = 10;
                i = cardText.toLowerCase().indexOf('hand size:');
                if (i == -1) {
                    throw new Error('Vanguard "Starting & Max Hand Size" info missing on card: ' + cardName);
                }
            }
        }
        var cardHand = Delphi.Trim(cardText.substr(i + j)).replace(/\./g, '');
        cardText = cardText.substr(0, i);
        //
        var HLTop = Iif(cardVersion == "old", 926, 668);
        var HLLeft = Iif(cardVersion == "old", 86, 53);
        var HLRight = Iif(cardVersion == "old", 642, 680);
        renderer = canvas.GetTextRenderer(Config.GeneratorFolder, HandLifeFont, HandLifeFont);
        try {
            renderer.Size = 24 - Iif(cardHand.length > 2, 6, 0);
            renderer.Color = Delphi.GPMakeColor(0x00, 0x00, 0x00);
            renderer.Text = cardHand;
            renderer.Measure({ "X": 0, "Y": 0 });
            renderer.Render(HLLeft, HLTop, GPStringAlignmentFar, GPStringAlignmentCenter);
            renderer.Size = 24 - Iif(cardLife.length > 2, 6, 0);
            renderer.Text = cardLife;
            renderer.Measure({ "X": 0, "Y": 0 });
            renderer.Render(HLRight, HLTop, GPStringAlignmentFar, GPStringAlignmentCenter);
        } finally {
            renderer.Free();
        }
        // Old Code //
        if (cardFlavor.length == 0) {
			renderer = canvas.GetTextRenderer(Config.GeneratorFolder, TextFont, TextFontI);
			try {
				var measure = { "X": 0, "Y": 0 };
				var legalLeft = Iif(cardVersion == "old", 139, 76);
				var legalRight = Iif(cardVersion == "old", 595, 661);
				var legalHeight = Iif(cardVersion == "old", 272, 256);
				var legalTop = Iif(cardVersion == "old", 670, 686);
				renderer.Size = 30;
				renderer.Color = Delphi.GPMakeColor(0x00, 0x00, 0x00);
				renderer.Text = Delphi.Trim(cardText);
				renderer.MaxWidth = legalRight - legalLeft;
				renderer.IsCenter = true;
				while (true) {
					renderer.Measure(measure);
					if (measure.Y <= legalHeight) {
						break;
					}
					renderer.Size -= 0.1;
					if (renderer.Size < 9) {
						throw new Error("Legal text does not fit on card: " + cardName);
					}
				}				
				renderer.Render(legalLeft, legalTop + ((legalHeight - measure.Y) / 2), GPStringAlignmentNear, GPStringAlignmentNear);
			} finally {
				renderer.Free();
			}
        } else {
			renderer = canvas.GetTextRenderer(Config.GeneratorFolder, TextFont, TextFontI);
			try {
				var measure = { "X": 0, "Y": 0 };
				var legalLeft = 76;
				var legalRight = 661;
				var legalHeight = Iif(cardVersion == "old", 117, 107);
				var legalTop = Iif(cardVersion == "old", 663, 684);
				renderer.Size = 30;
				renderer.Color = Delphi.GPMakeColor(0x00, 0x00, 0x00);
				renderer.Text = Delphi.Trim(cardText);
				renderer.MaxWidth = legalRight - legalLeft;
				renderer.IsCenter = true;
				while (true) {
					renderer.Measure(measure);
					if (measure.Y <= legalHeight) {
						break;
					}
					renderer.Size -= 0.1;
					if (renderer.Size < 9) {
						throw new Error("Legal text does not fit on card: " + cardName);
					}
				}				
				renderer.Render(legalLeft, legalTop + ((legalHeight - measure.Y) / 2), GPStringAlignmentNear, GPStringAlignmentNear);
			} finally {
				renderer.Free();				
			}			
			renderer = canvas.GetTextRenderer(Config.GeneratorFolder, TextFont, TextFontI);
			try {
				var measure = { "X": 0, "Y": 0 };
				var legalLeft = Iif(cardVersion == "old", 139, 76);
				var legalRight = Iif(cardVersion == "old", 595, 661);
				var legalHeight = Iif(cardVersion == "old", 139, 131);
				var legalTop = Iif(cardVersion == "old", 868, 865);
				renderer.Size = 27;
				renderer.Color = Delphi.GPMakeColor(0x00, 0x00, 0x00);
				renderer.Text = "#" + Delphi.Trim(cardFlavor);
				renderer.MaxWidth = legalRight - legalLeft;
				renderer.IsCenter = true;
				while (true) {
					renderer.Measure(measure);
					if (measure.Y <= legalHeight) {
						break;
					}
					renderer.Size -= 0.1;
					if (renderer.Size < 9) {
						throw new Error("Legal text does not fit on card: " + cardName);
					}
				}				
				renderer.Render(legalLeft, legalTop + ((legalHeight - measure.Y) / 2), GPStringAlignmentCenter, GPStringAlignmentNear);
			} finally {
				renderer.Free();				
			}
        }
		// CopyRight/Artist (old card only?)
		if ((cardArtist.length > 0) && (cardCopyright.length > 0)) {
            renderer = canvas.GetTextRenderer(Config.GeneratorFolder, "GaramondB.ttf", "GaramondB.ttf");
            try {
                renderer.Size = 16;
                renderer.Text = "Illus. " + cardArtist;
                renderer.Measure({ "X": 0, "Y": 0 });
                renderer.Color = Delphi.GPMakeColor(0x00, 0x00, 0x00);
                renderer.Render(370, Iif(cardVersion == "old", 958, 954), GPStringAlignmentCenter, GPStringAlignmentCenter);
                renderer.Color = Delphi.GPMakeColor(0xff, 0xff, 0xff);
                renderer.Render(368, Iif(cardVersion == "old", 956, 952), GPStringAlignmentCenter, GPStringAlignmentCenter);
            } finally {
                renderer.Free();
            }
            renderer = canvas.GetTextRenderer(Config.GeneratorFolder, "MPlantin.ttf", "MPlantin.ttf");
            try {
                renderer.Size = Iif(cardVersion == "old", 14, 12);
                renderer.Text = cardCopyright;
                renderer.Measure({ "X": 0, "Y": 0 });
                renderer.Color = Delphi.GPMakeColor(0x00, 0x00, 0x00);
                renderer.Render(368, Iif(cardVersion == "old", 974, 970), GPStringAlignmentCenter, GPStringAlignmentCenter);
            } finally {
                renderer.Free();
            }
        }
    } finally {
        canvas.Free();
    }
    return workBuffer;
}

function RenderFlipCard(Card, bigSeparator) {
    var workBuffer = CreateObject('TGPBitmap', 736, 1050, GPPixelFormat32bppARGB);
    var canvas = workBuffer.GetGraphics();
    try {
        canvas.SetInterpolationMode(GPInterpolationModeHighQualityBicubic);
        canvas.SetTextRenderingHint(GPTextRenderingHintAntiAlias);
		brush = CreateObject('TGPSolidBrush', Delphi.GPMakeColor(0x00, 0x00, 0x00));
		try {
			canvas.FillRectangle(brush, 0, 0, 736, 1050);
		} finally {
			brush.Free();
		}
		//
        var cardSet = Card.Values('Set');
        var cardName = Card.Values('Name');
        var cardVersion = Card.Values('Version');
        var cardColor = Card.Values('Color');
        var cardCost = PreCastingCost(Card.Values('Cost'));
        var cardType = Card.Values('Type').replace(/ - /g, '  ');
        var cardText = Delphi.Trim(Card.Values('Text')).replace(/ - /g, '  ');
        var cardPT = Card.Values('PT');
        var cardFlavor = Delphi.Trim(Card.Values('Flavor')).replace(/ - /g, '  ');
        var cardRarity = Card.Values('Rarity');
        var cardArtist = Card.Values('Artist');
        var cardNumber = Card.Values('Number');
        var cardMana = Card.Values('Mana');
        var cardLandColor = Delphi.Trim(Card.Values('LandColor'));
        var cardGuildName = Card.Values('GuildName');
        var cardAltFrame = Card.Values('AltFrame');
        var cardCopyright = Card.Values('Copyright').replace(/\(tm\)/g, '').replace(/\(c\)/g, '');
        var cardArt = Card.Values('Art');
        // Normalize LandColor
		cardLandColor = GetColors(PreCastingCost(cardLandColor));
		// Time to work with "cardText"...
		var cardTextArr = cardText.split(Iif(bigSeparator, '-----', '----'));
		cardText = Delphi.Trim(cardTextArr[0]);
		var cardTextFlipArr = Delphi.Trim(cardTextArr[1]).split('\n');
		var cardNameFlip = Delphi.Trim(cardTextFlipArr[0]);
		var cardTypeFlip = Delphi.Trim(cardTextFlipArr[1]);
		var cardPTFlip, cardTextFlipIndex;
		if (!cardTypeFlip.toLowerCase().Countain('creature')) {
			cardPTFlip = '';
			cardTextFlipIndex = 2;
		} else {
			cardPTFlip = Delphi.Trim(cardTextFlipArr[2]);
			cardTextFlipIndex = 3;
		}
		var cardTextFlip = Delphi.Trim(cardTextFlipArr[cardTextFlipIndex]);
		for (var Index = cardTextFlipIndex + 1; Index < cardTextFlipArr.length; Index++) {
			cardTextFlip += '\r\n' + Delphi.Trim(cardTextFlipArr[Index]);
		}
        // Art (Direct Handed Full...)
        var cardArtFull = '';
        if (cardArt.Countain('.full.')) {
            cardArtFull = cardArt;
            cardArt = ''; 
        }
        if (cardArt.length == 0) {
            var cleanTitle = cardName.toLowerCase().replace(/[:\/]/g, '');
            var list = Delphi.Directory(Config.ArtFolder + cardSet + "\\" + cleanTitle + "*.jpg");
            try {
                for(var i = list.Count; i > 0; i--) {
                    var j = i - 1;
                    var file = list.Strings(j).toLowerCase();
                    var chr = file.charAt(file.indexOf(cleanTitle) + cleanTitle.length);
                    if (file.Countain(".full.jpg")) {
                        cardArtFull = list.Strings(j);
                    }
                    if (file.Countain(".full.jpg") || !".(123456789".Countain(chr)) {
                        list.Delete(j);
                    }
                }
                if (list.Count > 0) {
                    cardArt = list.Strings(Delphi.Random(list.Count)); 
                }
            } finally {
                list.Free();    
            }
        }
        var ArtImage;
        ArtImage = GetDirectImage(cardArt);
        if (!ArtImage === false) {
            var ArtLeft = 42;
            var ArtRight = 694;
            var ArtTop = 313;
            var ArtBottom = 690;
            var ArtNewW = ArtRight - ArtLeft + 1;
            var ArtNewH = ArtBottom - ArtTop + 1;
            var ArtW = ArtNewW;
            var ArtH = ArtNewH;
            var saveState = canvas.Save();
            try {
                canvas.SetClip(ArtLeft, ArtTop, ArtW, ArtH);
                ArtH = ArtImage.GetHeight() * (ArtNewW / ArtImage.GetWidth());
                if (ArtH < ArtNewH) {
                    ArtH = ArtNewH;
                    ArtW = ArtImage.GetWidth() * (ArtNewH / ArtImage.GetHeight());
                }
                canvas.DrawImage(ArtImage, ArtLeft + ((ArtNewW - ArtW) / 2), ArtTop + ((ArtNewH - ArtH) / 2), ArtW, ArtH);
            } finally {
                canvas.Restore(saveState);
            }
            ArtImage.Free();
        } else if (cardArtFull.length > 0) {
            ArtImage = GetDirectImage(cardArtFull);
            if (!ArtImage === false) {
                canvas.DrawImage(ArtImage, 0, 0, 736, 1050);
                ArtImage.Free();
            }
        }
		// Select BackGround
        var symbols = GetColors(cardCost);
        var dualColor = GetDualColors(cardCost);
        if (cardColor.length == 0) {
            if (cardLandColor.length > 0) {
                cardColor = "Lnd";
            } else if (symbols.length > 1) {
                cardColor = "Gld";
            } else if (symbols.length > 0) {
                cardColor = symbols;
            } else {
                cardColor = "Art";
            }  
        }
        var image;
        var borderImage = false;
        var overlay = false; 
        if (cardColor == "Lnd") {
            if (cardLandColor.length = 0) {
                throw new Error("Land color missing for card: " + cardName);
            }
            if (cardLandColor.length > 2) {
                cardLandColor = "A";
            }
            image = GetImage("images\\eighth\\flip\\land\\" + cardLandColor + ".png"); 
            if (image === false) {
                throw new Error("Background image not found for land color: " + cardLandColor);
            } 
        } else {
            if (dualColor.length == 2) {
                image = GetImage("images\\eighth\\flip\\cards\\" + dualColor + ".png");
                if (image === false) {
                    throw new Error("Background image not found for color: " + dualColor);
                }
                if (Config.BlendGreyTitleAndType) {
                    overlay = GetImage("images\\eighth\\flip\\cards\\C-overlay.png");
                    if (overlay === false) {
                        throw new Error("Image not found: C-overlay.png");
                    } 
                }
            } else {
				image = GetImage("images\\eighth\\flip\\cards\\" + cardColor + ".png");
                if (image === false) {
                    throw new Error("Background image not found for color: " + cardColor);
                }
                if ((symbols.length == 2) && (Config.MultiColorDualBorder)) {
                    borderImage = GetImage("images\\eighth\\flip\\borders\\" + symbols + ".png");
                }
            }
        }
        // Bg
        canvas.DrawImage(image, 0, 0, 736, 1050);
        if (!borderImage === false) {
            canvas.DrawImage(borderImage, 0, 0, 736, 1050);
        }
        if (!overlay === false) {
            canvas.DrawImage(overlay, 0, 0, 736, 1050);
        }
		// UnFlip Draw
        // CastingCost...
        var costList = CreateObject("TStringList");
        try {
            var costSep = cardCost.indexOf("*");
            var SizeY;
            var DesY;
            var CCRight = 696;
            var CCTop = 46;
            if (costSep == -1) {
                costList.CommaText = cardCost.replace(/}{/g, '","').replace(/{/g, '"').replace(/}/g, '"');      
            } else {
                costList.CommaText = cardCost.substr(costSep + 1).replace(/}{/g, '","').replace(/{/g, '"').replace(/}/g, '"');          
            } 
            for (var i = 1; i <= costList.Count; i++) {
                var oneItem = costList.Strings(costList.Count - i);
                if ((oneItem.length == 2) && !"10,11,12,15,16".Countain(oneItem)) {
                    SizeY = 42;
                    DesY = -4;
                } else {
                    SizeY = 34;
                    DesY = 0;
                }
                image = GetSymbol(oneItem);
                SizeX = SizeY * (image.GetWidth() / image.GetHeight());
                CCRight -= SizeX;
                canvas.DrawImage(GetSymbol('shadow'), CCRight - 3, CCTop + DesY + 3, SizeX + 2, SizeY + 1);
                canvas.DrawImage(image, CCRight, CCTop + DesY, SizeX, SizeY);
                CCRight -= 3;    
            } 
            CCRight -= 5;
        } finally {
            costList.Free();
        }
        // CardTitle
        var fontTitle = "GaramondB.ttf";
        var renderer = canvas.GetTextRenderer(Config.GeneratorFolder, fontTitle, fontTitle);
        try {
            var measure = { "X":0, "Y":0 };
            var TitleLeft = 43;
            var TitleTop = 35;
            renderer.Size = 35;
            renderer.Color = Delphi.GPMakeColor(0x00, 0x00, 0x00);
            renderer.TextPowTou = cardName;
            while ((renderer.Measure(measure)) && (TitleLeft + measure.X >= CCRight)) {
                renderer.Size -= 0.1;
                if (renderer.Size < 8) {
                    throw new Error("Card title does not fit on card: " + cardName);
                }
            }
            renderer.TextPowTou = cardName;
            renderer.Render(TitleLeft, TitleTop, GPStringAlignmentNear, GPStringAlignmentNear);
        } finally {
            renderer.Free();
        }		
        // Power/Toughness
        if (cardPT.length > 0) {
            if (dualColor == "") {
                if ((cardColor.length > 1) && (cardColor != "Art") && (cardColor != "Gld")) {
                    image = GetImage("images\\eighth\\flip\\pt\\" + cardColor.substr(-1) + "u.png");
                } else {
                    image = GetImage("images\\eighth\\flip\\pt\\" + cardColor + "u.png");                
                }
            } else {
                var OneColor = dualColor.charAt(dualColor.length - 1);  
                image = GetImage("images\\eighth\\flip\\pt\\" + OneColor + "u.png");
            }
            if (image === false) {
                throw new Error("Power / toughness image not found for color: " + cardColor);
            }
            canvas.DrawImage(image, 0, 0, image.GetWidth(), image.GetHeight());
            // Render Text...
            var fontPT = "GaramondB.ttf";
            var renderer = canvas.GetTextRenderer(Config.GeneratorFolder, fontPT, fontPT);
            try {
                var measure = { "X":0, "Y":0 };
                renderer.Size = 36;
                renderer.Color = Delphi.GPMakeColor(0x00, 0x00, 0x00);
                renderer.TextPowTou = cardPT;
                while ((renderer.Measure(measure)) && (measure.X >= 118)) {
                    renderer.Size -= 0.1;
                    if (renderer.Size < 8) {
                        throw new Error("Power/toughness does not fit on card: " + cardName);
                    }
                }
                renderer.TextPowTou = cardPT;
                renderer.Render(650, 270, GPStringAlignmentCenter, GPStringAlignmentCenter);
            } finally {
                renderer.Free();
            }
        }
        if (cardPTFlip.length > 0) {
            if (dualColor == "") {
                if ((cardColor.length > 1) && (cardColor != "Art") && (cardColor != "Gld")) {
                    image = GetImage("images\\eighth\\flip\\pt\\" + cardColor.substr(0) + "d.png");
                } else {
                    image = GetImage("images\\eighth\\flip\\pt\\" + cardColor + "d.png");                
                }
            } else {
                var OneColor = dualColor.charAt(0);  
                image = GetImage("images\\eighth\\flip\\pt\\" + OneColor + "d.png");
            }
            if (image === false) {
                throw new Error("Power / toughness image not found for color: " + cardColor);
            }
            canvas.DrawImage(image, 0, 0, image.GetWidth(), image.GetHeight());
		}
        // Type
        var fontType = "GaramondB.ttf";
        renderer = canvas.GetTextRenderer(Config.GeneratorFolder, fontType, fontType);
        try {
            measure = { "X":0, "Y":0 };
            var TypeLeft = 43;
            var TypeTop = 273;
            renderer.Size = 30;
            renderer.Color = Delphi.GPMakeColor(0x00, 0x00, 0x00);
            renderer.TextPowTou = cardType;
            while ((renderer.Measure(measure)) && (TypeLeft + measure.X >= 570)) {
                renderer.Size -= 0.1;
                if (renderer.Size < 8) {
                    throw new Error("Card type does not fit on card: " + cardName);
                }
            }
            renderer.TextPowTou = cardType;
            renderer.Render(TypeLeft, TypeTop, GPStringAlignmentCenter, GPStringAlignmentNear);
        } finally {
            renderer.Free();
        }   
		// Text
		var textHeight = 130;
		var textLeft = 50;
		var textRight = 682;
		var textTop = 110;
		var textWidth = textRight - textLeft + 1;
		var textMeasure = { "X":0, "Y":0 };
		var textRenderer = canvas.GetTextRenderer(Config.GeneratorFolder, "MPlantin.ttf", "MPlantinI.ttf");
        try { 
			textRenderer.MaxWidth = textWidth;
            textRenderer.Size = 30; 
            textRenderer.Color = Delphi.GPMakeColor(0x00, 0x00, 0x00);
            textRenderer.Text = cardText.replace(/~this~/g, cardName); // UnFlipped Name
            while (true) {
				textRenderer.Measure(textMeasure);
                totalHeight = textMeasure.Y;
                var totalWidth = textMeasure.X; 
                if ((totalHeight < textHeight) && (totalWidth < textWidth)) {
					break;
				}   
                textRenderer.Size -= 0.1;
                if (textRenderer.Size < 8) {
					throw new Error("Text/Flavor doesn't fit on card: " + cardName);
				} 
			}
			textRenderer.IsCenter = (textRenderer.LineCount <= 2);
			textTop += (textHeight - totalHeight) / 2;
			textRenderer.Render(textLeft, textTop, GPStringAlignmentNear, GPStringAlignmentNear);
		} finally {
			textRenderer.Free();
        }  
        // Rarity
        var rarityTop = 1010;
        var rarityRight = 700;
        var rarityFolder = "images\\" + IfPre8th("preEighth", "eighth") + "\\rarity\\";
        image = GetImage(rarityFolder + cardSet + "_" + cardRarity + ".gif");
		if (image === false) {
			image = GetImage(rarityFolder + cardSet + "_" + cardRarity + ".png");
		}
        if (!image === false) {
            var SizeX = image.GetWidth();
            var SizeY = image.GetHeight();
            if (image.GetHeight() > 38) {
                SizeX = 38 * (image.GetWidth() / image.GetHeight()); 
                SizeY = 38;
            }
            if (SizeX > 66) {
                SizeX = 66;
                SizeY = 66 * (image.GetHeight() / image.GetWidth());
            }
            rarityRight -= SizeX;
            canvas.DrawImage(image, rarityRight, rarityTop - (SizeY / 2), SizeX, SizeY);
            rarityRight -= 5;
        }
		// CopyRight/Artist
        if ((cardArtist.length > 0) && (cardCopyright.length > 0)) {
            // Setup Color...
            var ArtCopyColor = 0x00;
            if ((cardAltFrame.length > 0) && ((cardColor == "B") || (cardColor == "R") || (cardColor == "Gld"))) {
                ArtCopyColor = 0xff; 
            } else if ((cardColor == "B") || (cardColor == "G") || (cardColor == "R") || (cardColor == "Lnd")) {
                ArtCopyColor = 0xff;
            }
            image = GetImage("images\\eighth\\brush_" + Iif(ArtCopyColor == 0, "0,0,0", "255,255,255") + ".png");
            canvas.DrawImage(image, 37, 994, 52, 12);
            renderer = canvas.GetTextRenderer(Config.GeneratorFolder, "GaramondB.ttf", "GaramondB.ttf");
            try {
                renderer.Size = 22;
                renderer.Text = cardArtist;
                renderer.Measure({ "X": 0, "Y": 0 });
                renderer.Color = Delphi.GPMakeColor(ArtCopyColor, ArtCopyColor, ArtCopyColor);
                renderer.Render(94, 1009, GPStringAlignmentFar, GPStringAlignmentNear);
            } finally {
                renderer.Free();
            }
            renderer = canvas.GetTextRenderer(Config.GeneratorFolder, "MPlantin.ttf", "MPlantin.ttf");
            try {
                renderer.Size = 13;
                renderer.Text = cardCopyright;
                renderer.Measure({ "X": 0, "Y": 0 });
                renderer.Color = Delphi.GPMakeColor(ArtCopyColor, ArtCopyColor, ArtCopyColor);
                renderer.Render(37, 1030, GPStringAlignmentFar, GPStringAlignmentNear);
            } finally {
                renderer.Free();
            }
        }
		// Turn, Flipped Side
		workBuffer.RotateFlip(GPRotate180FlipNone);
		// Flip Draw
        // CardTitle
        var fontTitle = "GaramondB.ttf";
        var renderer = canvas.GetTextRenderer(Config.GeneratorFolder, fontTitle, fontTitle);
        try {
            var measure = { "X":0, "Y":0 };
            var TitleLeft = 43;
            var TitleTop = 82;
            renderer.Size = 35;
            renderer.Color = Delphi.GPMakeColor(0x00, 0x00, 0x00);
            renderer.TextPowTou = cardNameFlip;
            while ((renderer.Measure(measure)) && (TitleLeft + measure.X >= 680)) {
                renderer.Size -= 0.1;
                if (renderer.Size < 8) {
                    throw new Error("Card title does not fit on card: " + cardName);
                }
            }
            renderer.TextPowTou = cardNameFlip;
            renderer.Render(TitleLeft, TitleTop, GPStringAlignmentNear, GPStringAlignmentNear);
        } finally {
            renderer.Free();
        }		
		//
		var TypeFlipRight = 680;
        if (cardPTFlip.length > 0) {			
            // Render Text...
            var fontPT = "GaramondB.ttf";
            var renderer = canvas.GetTextRenderer(Config.GeneratorFolder, fontPT, fontPT);
            try {
                var measure = { "X":0, "Y":0 };
                renderer.Size = 36;
                renderer.Color = Delphi.GPMakeColor(0x00, 0x00, 0x00);
                renderer.TextPowTou = cardPTFlip;
                while ((renderer.Measure(measure)) && (measure.X >= 118)) {
                    renderer.Size -= 0.1;
                    if (renderer.Size < 8) {
                        throw new Error("Power/toughness does not fit on card: " + cardName);
                    }
                }
                renderer.TextPowTou = cardPTFlip;
                renderer.Render(650, 315, GPStringAlignmentCenter, GPStringAlignmentCenter);
            } finally {
                renderer.Free();
            }
			TypeFlipRight = 580;
		}
        // Type
        var fontType = "GaramondB.ttf";
        renderer = canvas.GetTextRenderer(Config.GeneratorFolder, fontType, fontType);
        try {
            measure = { "X":0, "Y":0 };
            var TypeLeft = 43;
            var TypeTop = 318;
            renderer.Size = 30;
            renderer.Color = Delphi.GPMakeColor(0x00, 0x00, 0x00);
            renderer.TextPowTou = cardType;
            while ((renderer.Measure(measure)) && (TypeLeft + measure.X >= TypeFlipRight)) {
                renderer.Size -= 0.1;
                if (renderer.Size < 8) {
                    throw new Error("Card type does not fit on card: " + cardName);
                }
            }
            renderer.TextPowTou = cardType;
            renderer.Render(TypeLeft, TypeTop, GPStringAlignmentCenter, GPStringAlignmentNear);
        } finally {
            renderer.Free();
        }   
		// Text
		var textHeight = 130;
		var textLeft = 50;
		var textRight = 682;
		var textTop = 145;
		var textWidth = textRight - textLeft + 1;
		var textMeasure = { "X":0, "Y":0 };
		var textRenderer = canvas.GetTextRenderer(Config.GeneratorFolder, "MPlantin.ttf", "MPlantinI.ttf");
        try { 
			textRenderer.MaxWidth = textWidth;
            textRenderer.Size = 30; 
            textRenderer.Color = Delphi.GPMakeColor(0x00, 0x00, 0x00);
            textRenderer.Text = cardTextFlip.replace(/~this~/g, cardNameFlip); // Flipped Name
            while (true) {
				textRenderer.Measure(textMeasure);
                totalHeight = textMeasure.Y;
                var totalWidth = textMeasure.X; 
                if ((totalHeight < textHeight) && (totalWidth < textWidth)) {
					break;
				}   
                textRenderer.Size -= 0.1;
                if (textRenderer.Size < 8) {
					throw new Error("Text/Flavor doesn't fit on card: " + cardName);
				} 
			}
			textRenderer.IsCenter = (textRenderer.LineCount <= 2);
			textTop += (textHeight - totalHeight) / 2;
			textRenderer.Render(textLeft, textTop, GPStringAlignmentNear, GPStringAlignmentNear);
		} finally {
			textRenderer.Free();
        }  
		// UnTurn
		workBuffer.RotateFlip(GPRotate180FlipNone);
    } finally {
        canvas.Free();
    }
    return workBuffer;
}

function RenderPlaneswalkerCard(Card) {
    var workBuffer = CreateObject('TGPBitmap', 736, 1050, GPPixelFormat32bppARGB);
    var canvas = workBuffer.GetGraphics();
    try {
        canvas.SetInterpolationMode(GPInterpolationModeHighQualityBicubic);
        canvas.SetTextRenderingHint(GPTextRenderingHintAntiAlias);
		brush = CreateObject('TGPSolidBrush', Delphi.GPMakeColor(0x00, 0x00, 0x00));
		try {
			canvas.FillRectangle(brush, 0, 0, 736, 1050);
		} finally {
			brush.Free();
		}
		//
        var cardSet = Card.Values('Set');
        var cardName = Card.Values('Name');
        var cardVersion = Card.Values('Version');
        var cardColor = Card.Values('Color');
        var cardCost = PreCastingCost(Card.Values('Cost'));
        var cardType = Card.Values('Type').replace(/ - /g, '  ');
        var cardText = Delphi.Trim(Card.Values('Text')).replace(/~this~/g, cardName).replace(/ - /g, '  ');
        var cardPT = Card.Values('PT');
        var cardFlavor = Delphi.Trim(Card.Values('Flavor')).replace(/ - /g, '  ');
        var cardRarity = Card.Values('Rarity');
        var cardArtist = Card.Values('Artist');
        var cardNumber = Card.Values('Number');
        var cardMana = Card.Values('Mana');
        var cardLandColor = Delphi.Trim(Card.Values('LandColor'));
        var cardGuildName = Card.Values('GuildName');
        var cardAltFrame = Card.Values('AltFrame');
        var cardCopyright = Card.Values('Copyright').replace(/\(tm\)/g, '').replace(/\(c\)/g, '');
        var cardArt = Card.Values('Art');
		//
        var symbols = GetColors(cardCost);
        var dualColor = GetDualColors(cardCost);
        if (cardColor.length == 0) {
            if (cardLandColor.length > 0) {
                cardColor = "Lnd";
            } else if (symbols.length > 1) {
                cardColor = "Gld";
            } else if (symbols.length > 0) {
                cardColor = symbols;
            } else {
                cardColor = "Art";
            }  
        }
        // Art (Direct Handed Full...)
        var cardArtFull = '';
        if (cardArt.Countain('.full.')) {
            cardArtFull = cardArt;
            cardArt = ''; 
        }
        if (cardArt.length == 0) {
            var cleanTitle = cardName.toLowerCase().replace(/[:\/]/g, '');
            var list = Delphi.Directory(Config.ArtFolder + cardSet + "\\" + cleanTitle + "*.jpg");
            try {
                for(var i = list.Count; i > 0; i--) {
                    var j = i - 1;
                    var file = list.Strings(j).toLowerCase();
                    var chr = file.charAt(file.indexOf(cleanTitle) + cleanTitle.length);
                    if (file.Countain(".full.jpg")) {
                        cardArtFull = list.Strings(j);
                    }
                    if (file.Countain(".full.jpg") || !".(123456789".Countain(chr)) {
                        list.Delete(j);
                    }
                }
                if (list.Count > 0) {
                    cardArt = list.Strings(Delphi.Random(list.Count)); 
                }
            } finally {
                list.Free();    
            }
        }
        var ArtImage;
        ArtImage = GetDirectImage(cardArt);
        if (!ArtImage === false) {
            var ArtLeft = 0;
            var ArtRight = 736;
            var ArtTop = 0;
            var ArtBottom = 1050;
            var ArtNewW = ArtRight - ArtLeft + 1;
            var ArtNewH = ArtBottom - ArtTop + 1;
            var ArtW = ArtNewW;
            var ArtH = ArtNewH;
            var saveState = canvas.Save();
            try {
                canvas.SetClip(ArtLeft, ArtTop, ArtW, ArtH);
                ArtH = ArtImage.GetHeight() * (ArtNewW / ArtImage.GetWidth());
                if (ArtH < ArtNewH) {
                    ArtH = ArtNewH;
                    ArtW = ArtImage.GetWidth() * (ArtNewH / ArtImage.GetHeight());
                }
                canvas.DrawImage(ArtImage, ArtLeft + ((ArtNewW - ArtW) / 2), ArtTop + ((ArtNewH - ArtH) / 2), ArtW, ArtH);
            } finally {
                canvas.Restore(saveState);
            }
            ArtImage.Free();
        } else if (cardArtFull.length > 0) {
            ArtImage = GetDirectImage(cardArtFull);
            if (!ArtImage === false) {
                canvas.DrawImage(ArtImage, 0, 0, 736, 1050);
                ArtImage.Free();
            }
        }
		// Bg
		var image;
		if ((cardColor == "Gld") && (symbols.length == 2)) {
			image = GetImage("images\\planeswalker\\cards\\" + symbols + ".png"); 
		} else {
			image = GetImage("images\\planeswalker\\cards\\" + cardColor + ".png"); 
		}
        if (image === false) {
            throw new Error("Background image not found for color: " + cardLandColor);
        } 
        canvas.DrawImage(image, 0, 0, 736, 1050);
        // CastingCost...
        var costList = CreateObject("TStringList");
        try {
            var costSep = cardCost.indexOf("*");
            var SizeY;
            var DesY;
            var CCRight = 688;
            var CCTop = 26;
            if (costSep == -1) {
                costList.CommaText = cardCost.replace(/}{/g, '","').replace(/{/g, '"').replace(/}/g, '"');      
            } else {
                costList.CommaText = cardCost.substr(costSep + 1).replace(/}{/g, '","').replace(/{/g, '"').replace(/}/g, '"');          
            } 
            for (var i = 1; i <= costList.Count; i++) {
                var oneItem = costList.Strings(costList.Count - i);
                if ((oneItem.length == 2) && !"10,11,12,15,16".Countain(oneItem)) {
                    SizeY = 42;
                    DesY = -4;
                } else {
                    SizeY = 34;
                    DesY = 0;
                }
                image = GetSymbol(oneItem);
                SizeX = SizeY * (image.GetWidth() / image.GetHeight());
                CCRight -= SizeX;
                canvas.DrawImage(GetSymbol('shadow'), CCRight - 3, CCTop + DesY + 3, SizeX + 2, SizeY + 1);
                canvas.DrawImage(image, CCRight, CCTop + DesY, SizeX, SizeY);
                CCRight -= 3;    
            } 
            CCRight -= 5;
        } finally {
            costList.Free();
        }
        // CardTitle
        var fontTitle = "GaramondB.ttf";
        var renderer = canvas.GetTextRenderer(Config.GeneratorFolder, fontTitle, fontTitle);
        try {
            var measure = { "X":0, "Y":0 };
            var TitleLeft = 50;
            var TitleTop = 17;
            renderer.Size = 35;
            renderer.Color = Delphi.GPMakeColor(0x00, 0x00, 0x00);
            renderer.TextPowTou = cardName;
            while ((renderer.Measure(measure)) && (TitleLeft + measure.X >= CCRight)) {
                renderer.Size -= 0.1;
                if (renderer.Size < 8) {
                    throw new Error("Card title does not fit on card: " + cardName);
                }
            }
            renderer.TextPowTou = cardName;
            renderer.Render(TitleLeft, TitleTop, GPStringAlignmentNear, GPStringAlignmentNear);
        } finally {
            renderer.Free();
        }		
        // Rarity
        var rarityTop = 634;
        var rarityRight = 687;
        var rarityFolder = "images\\" + IfPre8th("preEighth", "eighth") + "\\rarity\\";
        image = GetImage(rarityFolder + cardSet + "_" + cardRarity + ".gif");
		if (image === false) {
			image = GetImage(rarityFolder + cardSet + "_" + cardRarity + ".png");
		}
        if (!image === false) {
            var SizeX = image.GetWidth();
            var SizeY = image.GetHeight();
            if (image.GetHeight() > 38) {
                SizeX = 38 * (image.GetWidth() / image.GetHeight()); 
                SizeY = 38;
            }
            if (SizeX > 66) {
                SizeX = 66;
                SizeY = 66 * (image.GetHeight() / image.GetWidth());
            }
            rarityRight -= SizeX;
            canvas.DrawImage(image, rarityRight, rarityTop - (SizeY / 2), SizeX, SizeY);
            rarityRight -= 5;
        }    
        // Type
        var fontType = "GaramondB.ttf";
        renderer = canvas.GetTextRenderer(Config.GeneratorFolder, fontType, fontType);
        try {
            measure = { "X":0, "Y":0 };
            var TypeLeft = 60;
            var TypeTop = 631;
            renderer.Size = 30;
            renderer.Color = Delphi.GPMakeColor(0x00, 0x00, 0x00);
            renderer.TextPowTou = cardType;
            while ((renderer.Measure(measure)) && (TypeLeft + measure.X >= rarityRight)) {
                renderer.Size -= 0.1;
                if (renderer.Size < 8) {
                    throw new Error("Card type does not fit on card: " + cardName);
                }
            }
            renderer.TextPowTou = cardType;
            renderer.Render(TypeLeft, TypeTop, GPStringAlignmentCenter, GPStringAlignmentNear);
        } finally {
            renderer.Free();
        }    
		// PT (Loyalty)
		var loyaltyImage = GetImage("images\\planeswalker\\loyalty\\LoyaltyBegin.png"); 
		if (loyaltyImage === false) {
			throw new Error("Loyalty symbol not found: LoyaltyBegin.png");
		}
		canvas.DrawImage(loyaltyImage, 665 - (loyaltyImage.GetWidth() / 2), 990 - (loyaltyImage.GetHeight() / 2), loyaltyImage.GetWidth(), loyaltyImage.GetHeight());
		//
        var fontPT = "GaramondB.ttf";
        var renderer = canvas.GetTextRenderer(Config.GeneratorFolder, fontPT, fontPT);
		var ptLeft = 663;
		var ptTop = 985;
        try {
            var measure = { "X":0, "Y":0 };
            renderer.Size = 36;
            renderer.Color = Delphi.GPMakeColor(0xff, 0xff, 0xff);
            renderer.TextPowTou = cardPT;
            while ((renderer.Measure(measure)) && (measure.X >= 70)) {
                renderer.Size -= 0.1;
                if (renderer.Size < 8) {
                    throw new Error("Loyalty does not fit on card: " + cardName);
                }
            }
            renderer.TextPowTou = cardPT;
            renderer.Render(ptLeft, ptTop, GPStringAlignmentCenter, GPStringAlignmentCenter);
        } finally {
            renderer.Free();
        }
		// Text (Tricky)
		var cardTextArr = Delphi.Trim(cardText).split('\n');
		if (cardTextArr.length == 3) {
			var loyaltyLeft = 62;
			var loyaltyTop = [721, 825, 921];
			var loyaltyTextLeft = 115;
			var loyaltyTextRight = 685;
			var loyaltyTextTop = [682, 778, 872];
			var loyaltyTextHeight = 80;
			var loyaltyTextWidth = loyaltyTextRight - loyaltyTextLeft + 1;
			for(var i = 0; i < cardTextArr.length; i++) {
				var cardTextLine = Delphi.Trim(cardTextArr[i]);
				var colonPos = cardTextLine.indexOf(':');
				if (colonPos > -1) {
					var loyalty = cardTextLine.substr(0, colonPos);
					var loyaltyValue = parseInt(loyalty);
					if (loyalty.charAt(0) == "+") {
						loyaltyValue = 1;
					} else if (loyalty.charAt(0) == "-") {
						loyaltyValue = -1;
					}
					var loyaltySymbol = "Loyalty";
					var loyaltyTextMoveY = 7;
					if (loyaltyValue > 0) {
						loyaltySymbol += "Up";
						loyaltyTextMoveY = 2;
					} else	if (loyaltyValue < 0) {
						loyaltySymbol += "Down";
						loyaltyTextMoveY = 10; 
					}
					var loyaltyImage = GetImage("images\\planeswalker\\loyalty\\" + loyaltySymbol + ".png"); 
					if (loyaltyImage === false) {
						throw new Error("Loyalty symbol not found: " + loyaltySymbol + ".png");
					}
					canvas.DrawImage(loyaltyImage, loyaltyLeft - (loyaltyImage.GetWidth() / 2), loyaltyTop[i] - (loyaltyImage.GetHeight() / 2), loyaltyImage.GetWidth(), loyaltyImage.GetHeight());
					// Draw Loyalty!
					var fontPT = "GaramondB.ttf";
					var renderer = canvas.GetTextRenderer(Config.GeneratorFolder, fontPT, fontPT);
					try {
						var measure = { "X":0, "Y":0 };
						renderer.Size = 25;
						renderer.Color = Delphi.GPMakeColor(0xff, 0xff, 0xff);
						renderer.TextPowTou = loyalty;
						while ((renderer.Measure(measure)) && (measure.X >= 70)) {
							renderer.Size -= 0.1;
							if (renderer.Size < 8) {
								throw new Error("Loyalty does not fit on card: " + cardName);
							}
						}
						renderer.Render(loyaltyLeft, loyaltyTop[i] - loyaltyTextMoveY, GPStringAlignmentCenter, GPStringAlignmentCenter);
					} finally {
						renderer.Free();
					}
				}
				// Rendering the Text!
				var textMeasure = { "X":0, "Y":0 };
				var textRenderer = canvas.GetTextRenderer(Config.GeneratorFolder, "MPlantin.ttf", "MPlantinI.ttf");
				try { 
					textRenderer.MaxWidth = loyaltyTextWidth;
					textRenderer.Size = 24; 
					textRenderer.Color = Delphi.GPMakeColor(0x00, 0x00, 0x00);
					textRenderer.Text = cardTextLine.substr(colonPos + 1);
					var totalHeight;
					var loyaltyTextHeightWork = loyaltyTextHeight;
					while (true) {
						textRenderer.Measure(textMeasure);
						totalHeight = textMeasure.Y;
						var totalWidth = textMeasure.X; 
						if ((totalHeight < loyaltyTextHeightWork) && (totalWidth < loyaltyTextWidth)) {
							break;
						}   
						textRenderer.Size -= 0.1;
						if (textRenderer.Size < 8) {
							throw new Error("Text/Flavor doesn't fit on card: " + cardName);
						} 
					}
					var textTop = loyaltyTextTop[i] + (loyaltyTextHeightWork - totalHeight) / 2;
					textRenderer.Render(loyaltyTextLeft, textTop, GPStringAlignmentNear, GPStringAlignmentNear);
				} finally {
					textRenderer.Free();
				}
			}
		} else {
			throw new Error("Planeswalker text must have 3 lines of text, one for each ability.");
		}
		// CopyRight/Artist
        if ((cardArtist.length > 0) && (cardCopyright.length > 0)) {
			//
			var brushLeft = 160;
			var brushTop = 1003;			
            // Setup Color...
            var ArtCopyColor = 0xFF; // White Text on Black Border...
            image = GetImage("images\\eighth\\brush_" + Iif(ArtCopyColor == 0, "0,0,0", "255,255,255") + ".png");
            canvas.DrawImage(image, brushLeft, brushTop, 52, 12);
            renderer = canvas.GetTextRenderer(Config.GeneratorFolder, "GaramondB.ttf", "GaramondB.ttf");
            try {
                renderer.Size = 22;
                renderer.Text = cardArtist;
                renderer.Measure({ "X": 0, "Y": 0 });
                renderer.Color = Delphi.GPMakeColor(ArtCopyColor, ArtCopyColor, ArtCopyColor);
                renderer.Render(brushLeft + 60, brushTop + 3, GPStringAlignmentCenter, GPStringAlignmentNear);
            } finally {
                renderer.Free();
            }
            renderer = canvas.GetTextRenderer(Config.GeneratorFolder, "MPlantin.ttf", "MPlantin.ttf");
            try {
                renderer.Size = 13;
                renderer.Text = cardCopyright;
                renderer.Measure({ "X": 0, "Y": 0 });
                renderer.Color = Delphi.GPMakeColor(ArtCopyColor, ArtCopyColor, ArtCopyColor);
                renderer.Render(brushLeft, brushTop + 35, GPStringAlignmentFar, GPStringAlignmentNear);
            } finally {
                renderer.Free();
            }
        }
    } finally {
        canvas.Free();
    }
    return workBuffer;
}

/* Render futureshifted frames... */
function RenderFutureCard(Card, isTextless) {
    var workBuffer = CreateObject('TGPBitmap', 736, 1050, GPPixelFormat32bppARGB);
    var canvas = workBuffer.GetGraphics();
    try {
        canvas.SetInterpolationMode(GPInterpolationModeHighQualityBicubic);
        canvas.SetTextRenderingHint(GPTextRenderingHintAntiAlias);
        //
        var cardSet = Card.Values('Set');
        var cardName = Card.Values('Name');
        var cardVersion = Card.Values('Version');
        var cardColor = Card.Values('Color');
        var cardCost = PreCastingCost(Card.Values('Cost'));
        var cardType = Card.Values('Type').replace(/ - /g, '  ');
        var cardText = Delphi.Trim(Card.Values('Text')).replace(/~this~/g, cardName).replace(/ - /g, '  ');
        var cardPT = Card.Values('PT');
        var cardFlavor = Delphi.Trim(Card.Values('Flavor')).replace(/ - /g, '  ');
        var cardRarity = Card.Values('Rarity');
        var cardArtist = Card.Values('Artist');
        var cardNumber = Card.Values('Number');
        var cardMana = Card.Values('Mana');
        var cardLandColor = Delphi.Trim(Card.Values('LandColor'));
        var cardGuildName = Card.Values('GuildName');
        var cardAltFrame = Card.Values('AltFrame');
        var cardCopyright = Card.Values('Copyright').replace(/\(tm\)/g, '').replace(/\(c\)/g, '');
        var cardArt = Card.Values('Art');
        // Normalize LandColor
		cardLandColor = GetColors(PreCastingCost(cardLandColor));
		//
        var symbols = GetColors(cardCost);
        var dualColor = GetDualColors(cardCost);
        if (cardColor.length == 0) {
            if (cardLandColor.length > 0) {
                cardColor = "Lnd";
            } else if (symbols.length > 1) {
                cardColor = "Gld";
            } else if (symbols.length > 0) {
                cardColor = symbols;
            } else {
                cardColor = "Art";
            }  
        }
		var overridePT = false;
        var image;
        var borderImage = false;
        var overlay = false; 
        if (cardColor == "Lnd") {
            if (cardLandColor.length = 0) {
                throw new Error("Land color missing for card: " + cardName);
            }
            if ((cardLandColor.length > 2) && (!Config.MultiColorLandFrame)) {
                cardLandColor = "A";
            }
            if (cardLandColor.length > 2) {
                cardLandColor = "A"; // FutureShifted: Up to two colors...
            }			
            image = GetImage("images\\eighth\\futureshifted\\land\\" + cardLandColor + ".png"); 
            if (image === false) {
                throw new Error("Background image not found for land color: " + cardLandColor);
            } 
			isTextless = false;
        } else {
            if (dualColor != "") {
                image = GetImage("images\\eighth\\futureshifted\\cards\\" + dualColor + ".png");
                if (image === false) {
                    throw new Error("Background image not found for color: " + dualColor);
                }
				isTextless = false;
            } else {
                image = GetImage("images\\eighth\\futureshifted\\" + Iif(isTextless, "textless", "cards") + "\\" + cardColor + ".png");
                if (image === false) {
                    throw new Error("Background image not found for color: " + cardColor);
                }
                if ((symbols.length == 2) && (Config.MultiColorDualBorder)) {
                    borderImage = GetImage("images\\eighth\\futureshifted\\borders\\" + symbols + ".png");
					overridePT = symbols.charAt(1); 
                }
            }
        }
        // Art (Direct Handed Full...)
        var cardArtFull = '';
        if (cardArt.Countain('.full.')) {
            cardArtFull = cardArt;
            cardArt = ''; 
        }
        if (cardArt.length == 0) {
            var cleanTitle = cardName.toLowerCase().replace(/[:\/]/g, '');
            var list = Delphi.Directory(Config.ArtFolder + cardSet + "\\" + cleanTitle + "*.jpg");
            try {
                for(var i = list.Count; i > 0; i--) {
                    var j = i - 1;
                    var file = list.Strings(j).toLowerCase();
                    var chr = file.charAt(file.indexOf(cleanTitle) + cleanTitle.length);
                    if (file.Countain(".full.jpg")) {
                        cardArtFull = list.Strings(j);
                    }
                    if (file.Countain(".full.jpg") || !".(123456789".Countain(chr)) {
                        list.Delete(j);
                    }
                }
                if (list.Count > 0) {
                    cardArt = list.Strings(Delphi.Random(list.Count)); 
                }
            } finally {
                list.Free();    
            }
        }
        var ArtImage;
        ArtImage = GetDirectImage(cardArt);
        if (!ArtImage === false) {
            var ArtLeft = 50;
            var ArtRight = 715;
            var ArtTop = 80;
            var ArtBottom = Iif(isTextless, 950, 715);
            var ArtNewW = ArtRight - ArtLeft + 1;
            var ArtNewH = ArtBottom - ArtTop + 1;
            var ArtW = ArtNewW;
            var ArtH = ArtNewH;
            var saveState = canvas.Save();
            try {
                canvas.SetClip(ArtLeft, ArtTop, ArtW, ArtH);
                ArtH = ArtImage.GetHeight() * (ArtNewW / ArtImage.GetWidth());
                if (ArtH < ArtNewH) {
                    ArtH = ArtNewH;
                    ArtW = ArtImage.GetWidth() * (ArtNewH / ArtImage.GetHeight());
                }
                canvas.DrawImage(ArtImage, ArtLeft + ((ArtNewW - ArtW) / 2), ArtTop + ((ArtNewH - ArtH) / 2), ArtW, ArtH);
            } finally {
                canvas.Restore(saveState);
            }
            ArtImage.Free();
        } else if (cardArtFull.length > 0) {
            ArtImage = GetDirectImage(cardArtFull);
            if (!ArtImage === false) {
                canvas.DrawImage(ArtImage, 0, 0, 736, 1050);
                ArtImage.Free();
            }
        }		
		// If a "Artifact" have mana cost with colors, render the superb border over it!
		// Examples:
		//  Reaper King = {2W}{2U}{2B}{2R}{2G}
		//  Sarcomite Myr = {2}{U}
		//  Tranguild Courier = {W}{U}{B}{R}{G}*{4}
		var cardColorByCost = "";
		if (cardColor == "Art") {
			cardColorByCost = symbols;
			if (cardColorByCost.length > 2) {
				cardColorByCost = "Gld";
			}
			if (cardColorByCost.length > 0) { 
				borderImage = GetImage("images\\eighth\\futureshifted\\borders\\" + cardColorByCost + ".png");
				if (borderImage === false) {
					throw new Error("Image not found: " + cardColorByCost + ".png");
				} 
				switch (cardColorByCost.length) {
				case 1: overridePT = cardColorByCost; break;
				case 2: overridePT = cardColorByCost.charAt(1); break;
				default: overridePT = 'Gld';
				}
			}
		}
        // Bg
        canvas.DrawImage(image, 0, 0, 736, 1050);
        if (!borderImage === false) {
            canvas.DrawImage(borderImage, 0, 0, 736, 1050);
        }
        if (!overlay === false) {
            canvas.DrawImage(overlay, 0, 0, 736, 1050);
        }
		var lastDualColor = Iif(dualColor != "", dualColor.charAt(dualColor.length - 1), "");
		var lastColorByCost = Iif(cardColorByCost != "", cardColorByCost.charAt(cardColorByCost.length - 1), "");
		if (
			(cardColor == "W") || 
			((cardColor == "Gld") && ((lastDualColor == "W") || (symbols.length > 2))) || 
			((cardColor == "Art") && (lastColorByCost != "B")) ||
			((cardColor == "Lnd") && ((cardLandColor == "W") || (cardLandColor == "A")))
		) {
			mainColor = Delphi.GPMakeColor(0x00, 0x00, 0x00);
			shadColor = Delphi.GPMakeColor(0xff, 0xff, 0xff);
		} else {
			mainColor = Delphi.GPMakeColor(0xff, 0xff, 0xff);
			shadColor = Delphi.GPMakeColor(0x00, 0x00, 0x00);
		}
        // Power/Toughness
        if (cardPT.length > 0) {
			if (!isTextless) {
				if (overridePT === false) {
					if (dualColor == "") {
						if ((cardColor.length > 1) && (cardColor != "Art") && (cardColor != "Gld")) {
							image = GetImage("images\\eighth\\futureshifted\\pt\\" + cardColor.substr(-1) + ".png");    
						} else {
							image = GetImage("images\\eighth\\futureshifted\\pt\\" + cardColor + ".png");
						}
					} else {
						var OneColor = dualColor.charAt(dualColor.length - 1);  
						image = GetImage("images\\eighth\\futureshifted\\pt\\" + OneColor + ".png");
					}
				} else {
					image = GetImage("images\\eighth\\futureshifted\\pt\\" + overridePT + ".png");
				}	
				if (image === false) {
					throw new Error("Power / toughness image not found for color: " + cardColor);
				}
				canvas.DrawImage(image, 0, 1050 - image.GetHeight(), image.GetWidth(), image.GetHeight());
			}
            // Render Text...
            var fontPT = 'GaramondB.ttf';
            var renderer = canvas.GetTextRenderer(Config.GeneratorFolder, fontPT, fontPT);
            try {
                var measure = { "X":0, "Y":0 };
                renderer.Size = 36;                
                renderer.TextPowTou = cardPT;
                while ((renderer.Measure(measure)) && (measure.X >= 118)) {
                    renderer.Size -= 0.1;
                    if (renderer.Size < 8) {
                        throw new Error("Power/toughness does not fit on card: " + cardName);
                    }
                }
				renderer.Color = shadColor;
                renderer.Render(629, 977, GPStringAlignmentCenter, GPStringAlignmentCenter);
				renderer.Color = mainColor;
                renderer.Render(627, 975, GPStringAlignmentCenter, GPStringAlignmentCenter);
            } finally {
                renderer.Free();
            }
        }
        // CastingCost...
        var costList = CreateObject("TStringList");
        try {
			/*
				Futureshifted use new symbols, and put the symbols.
				Numeric/X,Y,Z use globes and render text INSIDE it... (gosh)
				W/U/B/R/G/S+Hybrid Have it's ownz Symbols				
			*/
			
			if (isTextless) {
				var symbolPos = [
					{ "X": 116, "Y": 169 },
					{ "X": 86, "Y": 248 },
					{ "X": 84, "Y": 338 },
					{ "X": 82, "Y": 426 },
					{ "X": 86, "Y": 513 },
					{ "X": 88, "Y": 596 }
				];
			} else {
				var symbolPos = [
					{ "X": 111, "Y": 163 },
					{ "X": 80, "Y": 233 },
					{ "X": 69, "Y": 308 },
					{ "X": 69, "Y": 391 },
					{ "X": 84, "Y": 469 },
					{ "X": 129, "Y": 550 }
				];
			}
            var costSep = cardCost.indexOf("*");
            if (costSep == -1) {
                costList.CommaText = cardCost.replace(/}{/g, '","').replace(/{/g, '"').replace(/}/g, '"');      
            } else {
                costList.CommaText = cardCost.substr(costSep + 1).replace(/}{/g, '","').replace(/{/g, '"').replace(/}/g, '"');          
            } 
			if (costList.Count > 6) {
				throw new Error("Casting cost too long for card: " + cardName);
			}
			var fontMana = "GaramondB.ttf";
			var renderer = canvas.GetTextRenderer(Config.GeneratorFolder, fontMana, fontMana);
			var measure = { "X": 0, "Y": 0 };
			try {				
				for (var i = 0; i < costList.Count; i++) {
					var oneItem = costList.Strings(i);
					var posItem = symbolPos[i];
					if (!"W,U,B,R,G,S,BG,BR,GU,GW,RG,RW,UB,UR,WB,WU".Countain(oneItem)) {
						if ((cardColor == "Art") || (cardColor == "Lnd")) {
							image = GetFutureSymbol('C'); 
						} else {
							image = GetFutureSymbol('C' + cardColor); 
						}
						if (image === false) {
							throw new Error("Globe mana cost image not found for color: " + cardColor);
						}
						canvas.DrawImage(image, posItem.X - (image.GetWidth() / 2), posItem.Y - (image.GetHeight() / 2), image.GetWidth(), image.GetHeight());
						//
						renderer.Size = 35;
						renderer.TextPowTou = oneItem;
						renderer.Measure(measure);
						renderer.Color = Delphi.GPMakeColor(0x00, 0x00, 0x00);
						renderer.Render(posItem.X + 2, posItem.Y - 1, GPStringAlignmentCenter, GPStringAlignmentCenter);
						renderer.Color = Delphi.GPMakeColor(0xff, 0xff, 0xff);
						renderer.Render(posItem.X, posItem.Y - 3, GPStringAlignmentCenter, GPStringAlignmentCenter);
					} else {
						image = GetFutureSymbol(oneItem); 
						if (image === false) {
							throw new Error("Mana cost image not found for mana: " + oneItem);
						}
						canvas.DrawImage(image, posItem.X - (image.GetWidth() / 2), posItem.Y - (image.GetHeight() / 2), image.GetWidth(), image.GetHeight());
					}
				} 
			} finally {
				renderer.Free();
			}	
        } finally {
            costList.Free();
        }
		// Type Image (UpperLeft)
		var typeList = ['tribal', 'artifact', 'sorcery', 'land', 'instant', 'enchantment', 'creature'];
		var typeOfCard = cardType.toLowerCase();
		var curType = '';
		for(var i = 0; i < typeList.length; i++) {
			if (typeOfCard.Countain(typeList[i])) {
				if (curType != '') {
					curType = 'multiple';
				} else {
					curType = typeList[i];
				}
			}
		}
		if (curType) {
			var imageB = GetFutureSymbol(curType + 'B');
			var imageW = GetFutureSymbol(curType + 'W');
			if ((imageB === false) || (imageW === false)) {
				throw new Error("Type image not found for type: " + curType);
			}
			canvas.DrawImage(imageB, 60 - (imageB.GetWidth() / 2), 60 - (imageB.GetHeight() / 2), imageB.GetWidth(), imageB.GetHeight());
			canvas.DrawImage(imageW, 58 - (imageW.GetWidth() / 2), 58 - (imageW.GetHeight() / 2), imageW.GetWidth(), imageW.GetHeight());
		}
        // CardTitle
        var fontTitle = 'GaramondB.ttf';
        var renderer = canvas.GetTextRenderer(Config.GeneratorFolder, fontTitle, fontTitle);
        try {
            var measure = { "X":0, "Y":0 };
            var TitleLeft = 127;
            var TitleTop = 53;
            if (HaveTombStone(Card)) {
                image = GetImage('images\\preEighth\\tombstone.png');
                if (!image === false) { 
                    canvas.DrawImage(image, TitleLeft - 20, TitleTop, 32, 45);
                    TitleLeft += 20;
                }
            }
            renderer.Size = 35;
            renderer.Color = Delphi.GPMakeColor(0x00, 0x00, 0x00);
            renderer.TextPowTou = cardName;
            while ((renderer.Measure(measure)) && (TitleLeft + measure.X >= 675)) {
                renderer.Size -= 0.1;
                if (renderer.Size < 8) {
                    throw new Error("Card title does not fit on card: " + cardName);
                }
            }
            renderer.Color = shadColor;
            renderer.Render(TitleLeft + 2, TitleTop + 2, GPStringAlignmentNear, GPStringAlignmentNear);
            renderer.Color = mainColor;
            renderer.Render(TitleLeft, TitleTop, GPStringAlignmentNear, GPStringAlignmentNear);
        } finally {
            renderer.Free();
        }
        // Rarity
		var rarityX = 681;
		var rarityY = Iif(isTextless, 908, 629);
		
        var rarityFolder = "images\\" + IfPre8th("preEighth", "eighth") + "\\rarity\\";
        image = GetImage(rarityFolder + cardSet + "_" + cardRarity + ".gif");
		if (image === false) {
			image = GetImage(rarityFolder + cardSet + "_" + cardRarity + ".png");
		}
        if (!image === false) {
            var SizeX = image.GetWidth();
            var SizeY = image.GetHeight();
            if (image.GetHeight() > 45) {
                SizeX = 45 * (image.GetWidth() / image.GetHeight()); 
                SizeY = 45;
            }
            if (SizeX > 45) {
                SizeX = 45;
                SizeY = 45 * (image.GetHeight() / image.GetWidth());
            }
            canvas.DrawImage(image, rarityX - (SizeX / 2), rarityY - (SizeY / 2), SizeX, SizeY);
        }    
        var rarityRight = 620;
        // Type
        var fontType = 'GaramondB.ttf';
        renderer = canvas.GetTextRenderer(Config.GeneratorFolder, fontType, fontType);
        try {
			var isBasicLand = (cardType.toLowerCase().Countain("basic land") || cardType.toLowerCase().Countain("basic snow land"));
		
		
            measure = { "X":0, "Y":0 };
            var TypeLeft = 86;
            var TypeTop = Iif(isTextless, 900, 620);
            renderer.Size = 30;
            renderer.Color = Delphi.GPMakeColor(0x00, 0x00, 0x00);
            renderer.TextPowTou = cardType;
            while ((renderer.Measure(measure)) && (TypeLeft + measure.X >= rarityRight)) {
                renderer.Size -= 0.1;
                if (renderer.Size < 8) {
                    throw new Error("Card type does not fit on card: " + cardName);
                }
            }
            renderer.Color = shadColor;
            renderer.Render(TypeLeft + 2, TypeTop + 2, GPStringAlignmentCenter, GPStringAlignmentNear);
            renderer.Color = mainColor;
            renderer.Render(TypeLeft, TypeTop, GPStringAlignmentCenter, GPStringAlignmentNear);
        } finally {
            renderer.Free();
        }    
        // Guild/Land Symbol
        if (cardGuildName) {
            image = GetImage("images\\eighth\\guilds\\" + cardGuildName + ".png");
            if (image === false) {
                throw new Error("Guild sign image not found for: " + cardGuildName);
            }
            canvas.DrawImage(image, 373 - (image.GetWidth() / 2), 640, image.GetWidth(), image.GetHeight());
        }
        //
        if (cardType.toLowerCase().Countain("basic land") || cardType.toLowerCase().Countain("basic snow land")) {
            image = GetImage("images\\symbols\\land\\" + cardLandColor + ".png");
            if (image === false) {
                throw new Error("Basic land symbol not found for: " + cardLandColor);
            }
            canvas.DrawImage(image, 368 - (image.GetWidth() / 2), 950 - image.GetHeight(), image.GetWidth(), image.GetHeight());    
        } else if ((cardColor == "Lnd") && (cardLandColor.length == 2) && (cardText.length == 0)) {
            switch (Config.OutputDualLandSymbols) {
            case 1:
                image = GetImage("images\\symbols\\land\\" + cardLandColor + ".png");
                if (image === false) {
                    throw new Error("Merged land symbol not found for: " + cardLandColor);
                }
                canvas.DrawImage(image, 368 - (image.GetWidth() / 2), 940 - image.GetHeight(), image.GetWidth(), image.GetHeight());
                break;           
            case 2:
                var image1 = GetImage("images\\symbols\\land\\" + cardLandColor.charAt(0) + ".png"); 
                var image2 = GetImage("images\\symbols\\land\\" + cardLandColor.charAt(1) + ".png");
                if ((image1 === false) || (image2 === false)) {
                    throw new Error("Basic land symbol not found for: " + cardLandColor);
                }
				var baseLine = 960;
                var totalWidth = image1.GetWidth() + image2.GetWidth() + 10;
                var drawLeft = 368 - (totalWidth / 2);
                canvas.DrawImage(image1, drawLeft, baseLine - image1.GetHeight(), image1.GetWidth(), image1.GetHeight());
                drawLeft += image1.GetWidth() + 10;
                canvas.DrawImage(image2, drawLeft, baseLine - image2.GetHeight(), image2.GetWidth(), image2.GetHeight());
            }
        } else {
            var textHeight = 283;
            var textLeft = 62;
            var textRight = 676;
            var textTop = 656;
            var textWidth = textRight - textLeft + 1;
            var textMeasure = { "X":0, "Y":0 };
            var flavorMeasure = { "X":0, "Y":0 };
            var totalHeight = 0;
            if ((cardText.length > 0) && (cardFlavor.length > 0)) {
                var textRenderer = canvas.GetTextRenderer(Config.GeneratorFolder, "MPlantin.ttf", "MPlantinI.ttf");
                try { 
                    textRenderer.MaxWidth = textWidth;
                    textRenderer.Size = 30; 
                    textRenderer.Color = Delphi.GPMakeColor(0x00, 0x00, 0x00);
                    textRenderer.Text = cardText; 
                    var flavorRenderer = canvas.GetTextRenderer(Config.GeneratorFolder, "MPlantin.ttf", "MPlantinI.ttf");
                    try { 
                        flavorRenderer.MaxWidth = textWidth;
                        flavorRenderer.Size = 27; 
                        flavorRenderer.Color = Delphi.GPMakeColor(0x00, 0x00, 0x00);
                        flavorRenderer.Text = "#" + cardFlavor; 
                        while (true) {
                            textRenderer.Measure(textMeasure);
                            flavorRenderer.Measure(flavorMeasure);
                            totalHeight = textMeasure.Y + flavorMeasure.Y + 20;
                            //
                            var totalWidth = Max(textMeasure.X, flavorMeasure.X); 
                            if ((totalHeight < textHeight) && (totalWidth < textWidth)) {
                                break;
                            }   
                            textRenderer.Size -= 0.1;
                            flavorRenderer.Size -= 0.1;
                            if ((textRenderer.Size < 8) || (flavorRenderer.Size < 8)) {
                                throw new Error("Text/Flavor doesn't fit on card: " + cardName);
                            } 
                        }
                        textTop += (textHeight - totalHeight) / 2;
                        textRenderer.Render(textLeft, textTop, GPStringAlignmentNear, GPStringAlignmentNear);  
                        flavorRenderer.Render(textLeft, textTop + 20 + textMeasure.Y, GPStringAlignmentNear, GPStringAlignmentNear);
                    } finally {
                        flavorRenderer.Free();
                    }
                } finally {
                    textRenderer.Free();
                }
            }
            else if (cardText.length > 0) {
                var textRenderer = canvas.GetTextRenderer(Config.GeneratorFolder, "MPlantin.ttf", "MPlantinI.ttf");
                try { 
                    textRenderer.MaxWidth = textWidth;
                    textRenderer.Size = 30; 
                    textRenderer.Color = Delphi.GPMakeColor(0x00, 0x00, 0x00);
                    textRenderer.Text = cardText; 
                    while (true) {
                        textRenderer.Measure(textMeasure);
                        totalHeight = textMeasure.Y;
                        //
                        var totalWidth = textMeasure.X; 
                        if ((totalHeight < textHeight) && (totalWidth < textWidth)) {
                            break;
                        }   
                        textRenderer.Size -= 0.1;
                        if (textRenderer.Size < 8) {
                            throw new Error("Text/Flavor doesn't fit on card: " + cardName);
                        } 
                    }
                    textRenderer.IsCenter = (textRenderer.LineCount <= 2);
                    textTop += (textHeight - totalHeight) / 2;
                    textRenderer.Render(textLeft, textTop, GPStringAlignmentNear, GPStringAlignmentNear);
                } finally {
                    textRenderer.Free();
                }  
            }
            else if (cardFlavor.length > 0) {
                var flavorRenderer = canvas.GetTextRenderer(Config.GeneratorFolder, "MPlantin.ttf", "MPlantinI.ttf");
                try { 
                    flavorRenderer.MaxWidth = textWidth;
                    flavorRenderer.Size = 27; 
                    flavorRenderer.Color = Delphi.GPMakeColor(0x00, 0x00, 0x00);
                    flavorRenderer.Text = "#" + cardFlavor; 
                    while (true) {
                        flavorRenderer.Measure(flavorMeasure);
                        totalHeight = flavorMeasure.Y;
                        //
                        var totalWidth = flavorMeasure.X; 
                        if ((totalHeight < textHeight) && (totalWidth < textWidth)) {
                            break;
                        }   
                        flavorRenderer.Size -= 0.1;
                        if (flavorRenderer.Size < 8) {
                            throw new Error("Text/Flavor doesn't fit on card: " + cardName);
                        } 
                    }
                    flavorRenderer.IsCenter = (flavorRenderer.LineCount <= 2);
                    textTop += (textHeight - totalHeight) / 2;
                    flavorRenderer.Render(textLeft, textTop, GPStringAlignmentNear, GPStringAlignmentNear);
                } finally {
                    flavorRenderer.Free();
                }  
            }
        }
        // Artist+Copyright
        if ((cardArtist.length > 0) && (cardCopyright.length > 0)) {
            // Setup Color...
			var OneColor = dualColor.charAt(dualColor.length - 1);  
            var ArtCopyColor = 0x00;
            if ("BGR".Countain(cardColor) || "BGR".Countain(OneColor) || (cardColor == "Lnd")) {
                ArtCopyColor = 0xff;
            }
			if (cardPT.length > 0) {
				copyArtX = 555;
			} else {
				copyArtX = 680;
			}
			copyArtY = 978;			
            renderer = canvas.GetTextRenderer(Config.GeneratorFolder, "GaramondB.ttf", "GaramondB.ttf");
            try {
				var measure = { "X": 0, "Y": 0 };
                renderer.Size = 22;
                renderer.Text = cardArtist;
                renderer.Measure(measure);
                renderer.Color = Delphi.GPMakeColor(ArtCopyColor, ArtCopyColor, ArtCopyColor);
                renderer.Render(copyArtX, copyArtY, GPStringAlignmentFar, GPStringAlignmentFar);
				//
				image = GetImage("images\\eighth\\brush_" + Iif(ArtCopyColor == 0, "0,0,0", "255,255,255") + ".png");
				canvas.DrawImage(image, copyArtX - measure.X - 52 - 10, copyArtY - measure.Y + 8, 52, 12);
            } finally {
                renderer.Free();
            }
            renderer = canvas.GetTextRenderer(Config.GeneratorFolder, "MPlantin.ttf", "MPlantin.ttf");
            try {
				var measure = { "X": 0, "Y": 0 };
                renderer.Size = 13;
                renderer.Text = cardCopyright;
                renderer.Measure(measure);
                renderer.Color = Delphi.GPMakeColor(ArtCopyColor, ArtCopyColor, ArtCopyColor);
                renderer.Render(copyArtX, copyArtY + measure.Y + 6, GPStringAlignmentFar, GPStringAlignmentFar);
            } finally {
                renderer.Free();
            }
        }
    } finally {
        canvas.Free();
    }
    return workBuffer;
}

/* Render a simple and one card info... */
function RenderTokenCard(Card) {
    var workBuffer = CreateObject('TGPBitmap', 736, 1050, GPPixelFormat32bppARGB);
    var canvas = workBuffer.GetGraphics();
    try {
        canvas.SetInterpolationMode(GPInterpolationModeHighQualityBicubic);
        canvas.SetTextRenderingHint(GPTextRenderingHintAntiAlias);
        //
        var cardSet = Card.Values('Set');
        var cardName = Card.Values('Name');
        var cardVersion = Card.Values('Version');
        var cardColor = Card.Values('Color');
        var cardCost = PreCastingCost(Card.Values('Cost'));
        var cardType = Card.Values('Type').replace(/ - /g, '  ');
        var cardText = Delphi.Trim(Card.Values('Text')).replace(/~this~/g, cardName).replace(/ - /g, '  ');
        var cardPT = Card.Values('PT');
        var cardFlavor = Delphi.Trim(Card.Values('Flavor')).replace(/ - /g, '  ');
        var cardRarity = Card.Values('Rarity');
        var cardArtist = Card.Values('Artist');
        var cardNumber = Card.Values('Number');
        var cardMana = Card.Values('Mana');
        var cardLandColor = Delphi.Trim(Card.Values('LandColor'));
        var cardGuildName = Card.Values('GuildName');
        var cardAltFrame = Card.Values('AltFrame');
        var cardCopyright = Card.Values('Copyright').replace(/\(tm\)/g, '').replace(/\(c\)/g, '');
        var cardArt = Card.Values('Art');
        // Normalize LandColor
		cardLandColor = GetColors(PreCastingCost(cardLandColor));
		//
        var symbols = GetColors(cardCost);
        var dualColor = GetDualColors(cardCost);
        if (cardColor.length == 0) {
            if (cardLandColor.length > 0) {
                cardColor = "Lnd";
            } else if (symbols.length > 1) {
                cardColor = "Gld";
            } else if (symbols.length > 0) {
                cardColor = symbols;
            } else {
                cardColor = "Art";
            }  
        }
        var image;
        var borderImage = false;
        var overlay = false; 
		var folder = IfPre8th('preEighth', 'eighth');
        if (cardColor == "Lnd") {
            image = GetImage("images\\" + folder + "\\token\\cards\\C.png"); 
            if (image === false) {
                throw new Error("Background image not found for color: Lnd/C");
            } 
        } else {
            if (dualColor != "") {
                image = GetImage("images\\" + folder + "\\token\\cards\\" + dualColor + ".png");
                if (image === false) {
                    throw new Error("Background image not found for color: " + dualColor);
                }
            } else {
                image = GetImage("images\\" + folder + "\\token\\cards\\" + cardColor + ".png");
                if (image === false) {
                    throw new Error("Background image not found for color: " + cardColor);
                }
                if ((symbols.length == 2) && (Config.MultiColorDualBorder)) {
                    borderImage = GetImage("images\\" + folder + "\\token\\borders\\" + symbols + ".png");
                }
            }
        }
        // Art (Direct Handed Full...)
        var cardArtFull = '';
        if (cardArt.Countain('.full.')) {
            cardArtFull = cardArt;
            cardArt = ''; 
        }
        if (cardArt.length == 0) {
            var cleanTitle = cardName.toLowerCase().replace(/[:\/]/g, '');
            var list = Delphi.Directory(Config.ArtFolder + cardSet + "\\" + cleanTitle + "*.jpg");
            try {
                for(var i = list.Count; i > 0; i--) {
                    var j = i - 1;
                    var file = list.Strings(j).toLowerCase();					
					var chrPos = file.indexOf(cleanTitle) + cleanTitle.length;
                    var chr = file.charAt(chrPos);
                    if (file.indexOf(".full.jpg") == chrPos) { // .full.jpg must be at end of name...
                        cardArtFull = list.Strings(j);
                    }
                    if (file.Countain(".full.jpg") || !".(123456789".Countain(chr)) {
                        list.Delete(j);
                    }
                }
                if (list.Count > 0) {
                    cardArt = list.Strings(Delphi.Random(list.Count)); 
                }
            } finally {
                list.Free();    
            }
        }
        var ArtImage;
        ArtImage = GetDirectImage(cardArt);
        if (!ArtImage === false) {
            var ArtLeft = IfPre8th(67, 42);
            var ArtRight = IfPre8th(669, 694);
            var ArtTop = IfPre8th(82, 104);
            var ArtBottom = IfPre8th(660, 705);
            var ArtNewW = ArtRight - ArtLeft + 1;
            var ArtNewH = ArtBottom - ArtTop + 1;
            var ArtW = ArtNewW;
            var ArtH = ArtNewH;
            var saveState = canvas.Save();
            try {
                canvas.SetClip(ArtLeft, ArtTop, ArtW, ArtH);
                ArtH = ArtImage.GetHeight() * (ArtNewW / ArtImage.GetWidth());
                if (ArtH < ArtNewH) {
                    ArtH = ArtNewH;
                    ArtW = ArtImage.GetWidth() * (ArtNewH / ArtImage.GetHeight());
                }
                canvas.DrawImage(ArtImage, ArtLeft + ((ArtNewW - ArtW) / 2), ArtTop + ((ArtNewH - ArtH) / 2), ArtW, ArtH);
            } finally {
                canvas.Restore(saveState);
            }
            ArtImage.Free();
        } else if (cardArtFull.length > 0) {
            ArtImage = GetDirectImage(cardArtFull);
            if (!ArtImage === false) {
                canvas.DrawImage(ArtImage, 0, 0, 736, 1050);
                ArtImage.Free();
            }
        }
        // Bg
        canvas.DrawImage(image, 0, 0, 736, 1050);
        if (!borderImage === false) {
            canvas.DrawImage(borderImage, 0, 0, 736, 1050);
        }
        if (!overlay === false) {
            canvas.DrawImage(overlay, 0, 0, 736, 1050);
        }
		//
		var corTintaPreta = IfPre8th("W", "W,U,B,R,G,Lnd,Art");
        // Power/Toughness
        if (cardPT.length > 0) {
			//
			var PTTop = IfPre8th(955, 980);
			var PTLeft = IfPre8th(607, 622);
            // Render Text...
            var fontPT = IfPre8th("MPlantinB.ttf", "GaramondB.ttf");
            var renderer = canvas.GetTextRenderer(Config.GeneratorFolder, fontPT, fontPT);
            try {
                var measure = { "X":0, "Y":0 };
                renderer.Size = 36;
                renderer.Color = Delphi.GPMakeColor(0x00, 0x00, 0x00);
                renderer.TextPowTou = cardPT;
                while ((renderer.Measure(measure)) && (measure.X >= 100)) {
                    renderer.Size -= 0.1;
                    if (renderer.Size < 8) {
                        throw new Error("Power/toughness does not fit on card: " + cardName);
                    }
                }
                renderer.Color = Delphi.GPMakeColor(0x00, 0x00, 0x00);
				if (!corTintaPreta.Countain(cardColor.charAt(0))) {
					renderer.Render(PTLeft + 2, PTTop + 2, GPStringAlignmentCenter, GPStringAlignmentCenter);
					renderer.Color = Delphi.GPMakeColor(0xff, 0xff, 0xff);             
				}
                renderer.TextPowTou = cardPT;
                renderer.Render(PTLeft, PTTop, GPStringAlignmentCenter, GPStringAlignmentCenter);
            } finally {
                renderer.Free();
            }
        }
        // CardTitle
        var fontTitle = IfPre8th("Pre8th.ttf", "GaramondB.ttf");
        var renderer = canvas.GetTextRenderer(Config.GeneratorFolder, fontTitle, fontTitle);
        try {
			var titleWidth = 600;
            var measure = { "X":0, "Y":0 };
            var TitleLeft = 368;
            var TitleTop = IfPre8th(65, 65);
			if (!Config.IsPre8thFrame) {
				var image = GetImage("images\\" + folder + "\\token\\Title.png");
				if (image === false) {
					throw new Error("Card title image not found.");
				}
				titleWidth = image.GetWidth();
				canvas.DrawImage(image, TitleLeft - (titleWidth / 2), TitleTop - (image.GetHeight() / 2) + 6, titleWidth, image.GetHeight());
			}		
            renderer.Size = 35;
            renderer.TextPowTou = cardName;
            while ((renderer.Measure(measure)) && (measure.X >= titleWidth)) {
                renderer.Size -= 0.1;
                if (renderer.Size < 8) {
                    throw new Error("Card title does not fit on card: " + cardName);
                }
            }
            renderer.Color = Delphi.GPMakeColor(0x00, 0x00, 0x00);
            renderer.Render(TitleLeft + 2, TitleTop + 2, GPStringAlignmentCenter, GPStringAlignmentCenter);
            renderer.Color = Delphi.GPMakeColor(0xff, 0xff, 0xff);             
            renderer.TextPowTou = cardName;
            renderer.Render(TitleLeft, TitleTop, GPStringAlignmentCenter, GPStringAlignmentCenter);
        } finally {
            renderer.Free();
        }
        // Rarity
        var rarityTop = IfPre8th(724, 734);
        var rarityRight = IfPre8th(660, 680);
        var rarityFolder = "images\\" + IfPre8th("preEighth", "eighth") + "\\rarity\\";
        image = GetImage(rarityFolder + cardSet + "_" + cardRarity + ".gif");
		if (image === false) {
			image = GetImage(rarityFolder + cardSet + "_" + cardRarity + ".png");
		} 
        if (!image === false) {
            var SizeX = image.GetWidth();
            var SizeY = image.GetHeight();
            if (image.GetHeight() > 38) {
                SizeX = 38 * (image.GetWidth() / image.GetHeight()); 
                SizeY = 38;
            }
            if (SizeX > 66) {
                SizeX = 66;
                SizeY = 66 * (image.GetHeight() / image.GetWidth());
            }
            rarityRight -= SizeX;
            canvas.DrawImage(image, rarityRight, rarityTop - (SizeY / 2), SizeX, SizeY);
            rarityRight -= 5;
        }    
        // Type
        var fontType = IfPre8th("MPlantin.ttf", "GaramondB.ttf");
        renderer = canvas.GetTextRenderer(Config.GeneratorFolder, fontType, fontType);
        try {
            measure = { "X":0, "Y":0 };
            var TypeLeft = IfPre8th(90, 62);
            var TypeTop = rarityTop - IfPre8th(6, 4);
            renderer.Size = 30;
            renderer.Color = Delphi.GPMakeColor(0x00, 0x00, 0x00);
            renderer.TextPowTou = cardType;
            while ((renderer.Measure(measure)) && (TypeLeft + measure.X >= rarityRight)) {
                renderer.Size -= 0.1;
                if (renderer.Size < 8) {
                    throw new Error("Card type does not fit on card: " + cardName);
                }
            }
            renderer.Color = Delphi.GPMakeColor(0x00, 0x00, 0x00);
			if (!corTintaPreta.Countain(cardColor.charAt(0))) {
				renderer.Render(TypeLeft + 2, TypeTop + 2, GPStringAlignmentCenter, GPStringAlignmentNear);
				renderer.Color = Delphi.GPMakeColor(0xff, 0xff, 0xff);             
			}
            renderer.TextPowTou = cardType;
            renderer.Render(TypeLeft, TypeTop, GPStringAlignmentCenter, GPStringAlignmentNear);
        } finally {
            renderer.Free();
        }    
        // Guild/Land Symbol
        if (cardGuildName) {
            image = GetImage("images\\eighth\\guilds\\" + cardGuildName + ".png");
            if (image === false) {
                throw new Error("Guild sign image not found for: " + cardGuildName);
            }
            canvas.DrawImage(image, 373 - (image.GetWidth() / 2), IfPre8th(630, 650), image.GetWidth(), image.GetHeight());
        }
        //
        var textHeight = IfPre8th(150, 150);
        var textLeft = IfPre8th(120, 60);
        var textRight = IfPre8th(622, 680);
        var textTop = IfPre8th(750, 785);
        var textWidth = textRight - textLeft + 1;
        var textMeasure = { "X":0, "Y":0 };
        var flavorMeasure = { "X":0, "Y":0 };
        var totalHeight = 0;
        if ((cardText.length > 0) && (cardFlavor.length > 0)) {
            var textRenderer = canvas.GetTextRenderer(Config.GeneratorFolder, "MPlantin.ttf", "MPlantinI.ttf");
            try { 
                textRenderer.MaxWidth = textWidth;
                textRenderer.Size = 30; 
                textRenderer.Color = Delphi.GPMakeColor(0x00, 0x00, 0x00);
                textRenderer.Text = cardText; 
                var flavorRenderer = canvas.GetTextRenderer(Config.GeneratorFolder, "MPlantin.ttf", "MPlantinI.ttf");
                try { 
                    flavorRenderer.MaxWidth = textWidth;
                    flavorRenderer.Size = 27; 
                    flavorRenderer.Color = Delphi.GPMakeColor(0x00, 0x00, 0x00);
                    flavorRenderer.Text = "#" + cardFlavor; 
                    while (true) {
                        textRenderer.Measure(textMeasure);
                        flavorRenderer.Measure(flavorMeasure);
                        totalHeight = textMeasure.Y + flavorMeasure.Y + 20;
                        //
                        var totalWidth = Max(textMeasure.X, flavorMeasure.X); 
                        if ((totalHeight < textHeight) && (totalWidth < textWidth)) {
                            break;
                        }   
                        textRenderer.Size -= 0.1;
                        flavorRenderer.Size -= 0.1;
                        if ((textRenderer.Size < 8) || (flavorRenderer.Size < 8)) {
                            throw new Error("Text/Flavor doesn't fit on card: " + cardName);
                        } 
                    }
                    textTop += (textHeight - totalHeight) / 2;
                    textRenderer.Render(textLeft, textTop, GPStringAlignmentNear, GPStringAlignmentNear);  
                    flavorRenderer.Render(textLeft, textTop + 20 + textMeasure.Y, GPStringAlignmentNear, GPStringAlignmentNear);
                } finally {
                    flavorRenderer.Free();
                }
            } finally {
                textRenderer.Free();
            }
        }
        else if (cardText.length > 0) {
            var textRenderer = canvas.GetTextRenderer(Config.GeneratorFolder, "MPlantin.ttf", "MPlantinI.ttf");
            try { 
                textRenderer.MaxWidth = textWidth;
                textRenderer.Size = 30; 
                textRenderer.Color = Delphi.GPMakeColor(0x00, 0x00, 0x00);
                textRenderer.Text = cardText; 
                while (true) {
                    textRenderer.Measure(textMeasure);
                    totalHeight = textMeasure.Y;
                    //
                    var totalWidth = textMeasure.X; 
                    if ((totalHeight < textHeight) && (totalWidth < textWidth)) {
                        break;
                    }   
                    textRenderer.Size -= 0.1;
                    if (textRenderer.Size < 8) {
                        throw new Error("Text/Flavor doesn't fit on card: " + cardName);
                    } 
                }
                textRenderer.IsCenter = (textRenderer.LineCount <= 2);
                textTop += (textHeight - totalHeight) / 2;
                textRenderer.Render(textLeft, textTop, GPStringAlignmentNear, GPStringAlignmentNear);
            } finally {
                textRenderer.Free();
            }  
        }
        else if (cardFlavor.length > 0) {
            var flavorRenderer = canvas.GetTextRenderer(Config.GeneratorFolder, "MPlantin.ttf", "MPlantinI.ttf");
            try { 
                flavorRenderer.MaxWidth = textWidth;
                flavorRenderer.Size = 27; 
                flavorRenderer.Color = Delphi.GPMakeColor(0x00, 0x00, 0x00);
                flavorRenderer.Text = "#" + cardFlavor; 
                while (true) {
                    flavorRenderer.Measure(flavorMeasure);
                    totalHeight = flavorMeasure.Y;
                    //
                    var totalWidth = flavorMeasure.X; 
                    if ((totalHeight < textHeight) && (totalWidth < textWidth)) {
                        break;
                    }   
                    flavorRenderer.Size -= 0.1;
                    if (flavorRenderer.Size < 8) {
                        throw new Error("Text/Flavor doesn't fit on card: " + cardName);
                    } 
                }
                flavorRenderer.IsCenter = (flavorRenderer.LineCount <= 2);
                textTop += (textHeight - totalHeight) / 2;
                flavorRenderer.Render(textLeft, textTop, GPStringAlignmentNear, GPStringAlignmentNear);
            } finally {
                flavorRenderer.Free();
            }  
        }
        // Artist+Copyright
        if ((cardArtist.length > 0) && (cardCopyright.length > 0)) {
            if (Config.IsPre8thFrame) {
                renderer = canvas.GetTextRenderer(Config.GeneratorFolder, "GaramondB.ttf", "GaramondB.ttf");
                try {
                    renderer.Size = 20;
                    renderer.Text = "Illus. " + cardArtist;
                    renderer.Measure({ "X": 0, "Y": 0 });
                    renderer.Color = Delphi.GPMakeColor(0x00, 0x00, 0x00);
                    renderer.Render(270, 942, GPStringAlignmentCenter, GPStringAlignmentCenter);
                    renderer.Color = Delphi.GPMakeColor(0xff, 0xff, 0xff);
                    renderer.Render(268, 940, GPStringAlignmentCenter, GPStringAlignmentCenter);
                } finally {
                    renderer.Free();
                }
                renderer = canvas.GetTextRenderer(Config.GeneratorFolder, "MPlantin.ttf", "MPlantin.ttf");
                try {
                    renderer.Size = 14;
                    renderer.Text = cardCopyright;
                    renderer.Measure({ "X": 0, "Y": 0 });
                    renderer.Color = Iif(cardColor == "W", Delphi.GPMakeColor(0x00, 0x00, 0x00), Delphi.GPMakeColor(0xff, 0xff, 0xff));
                    renderer.Render(268, 965, GPStringAlignmentCenter, GPStringAlignmentCenter);
                } finally {
                    renderer.Free();
                }
            } else {
                // Setup Color...
                var ArtCopyColor = 0x00;
				//
                if (!"W,U".Countain(cardColor.charAt(0))) {
                    ArtCopyColor = 0xff;
                }
                image = GetImage("images\\eighth\\brush_" + Iif(ArtCopyColor == 0, "0,0,0", "255,255,255") + ".png");
                canvas.DrawImage(image, 47, 979, 52, 12);
                //
                renderer = canvas.GetTextRenderer(Config.GeneratorFolder, "GaramondB.ttf", "GaramondB.ttf");
                try {
                    renderer.Size = 22;
                    renderer.Text = cardArtist;
                    renderer.Measure({ "X": 0, "Y": 0 });
                    renderer.Color = Delphi.GPMakeColor(ArtCopyColor, ArtCopyColor, ArtCopyColor);
                    renderer.Render(104, 997, GPStringAlignmentFar, GPStringAlignmentNear);
                } finally {
                    renderer.Free();
                }
                //
                renderer = canvas.GetTextRenderer(Config.GeneratorFolder, "MPlantin.ttf", "MPlantin.ttf");
                try {
                    renderer.Size = 13;
                    renderer.Text = cardCopyright;
                    renderer.Measure({ "X": 0, "Y": 0 });
                    renderer.Color = Delphi.GPMakeColor(ArtCopyColor, ArtCopyColor, ArtCopyColor);
                    renderer.Render(47, 1015, GPStringAlignmentFar, GPStringAlignmentNear);
                } finally {
                    renderer.Free();
                }
            }
        }
    } finally {
        canvas.Free();
    }
    return workBuffer;
}
